# SPDX-License-Identifier: Apache-2.0
# Copyright © 2025 Au-Zone Technologies. All Rights Reserved.

from dataclasses import dataclass

from pycdr2 import IdlStruct
from pycdr2.types import float32, int8, sequence, uint32

from . import default_field
from .builtin_interfaces import Time
from .geometry_msgs import (Point, Pose, PoseStamped, PoseWithCovariance,
                            TwistWithCovariance)
from .std_msgs import Header


@dataclass
class GridCells(IdlStruct, typename='nav_msgs/GridCells'):
    """
    An array of cells in a 2D grid
    """
    header: Header = default_field(Header)

    cell_width: float32 = 0
    """
    Width of each cell
    """

    cell_height: float32 = 0
    """
    Height of each cell
    """

    cells: sequence[Point] = default_field([])
    """
    Each cell is represented by the Point at the center of the cell
    """


@dataclass
class MapMetaData(IdlStruct, typename='nav_msgs/MapMetaData'):
    """
    This hold basic information about the characteristics of the OccupancyGrid
    """
    map_load_time: Time = default_field(Time)
    """
    The time at which the map was loaded
    """

    resolution: float32 = 0
    """
    The map resolution [m/cell]
    """

    width: uint32 = 0
    """
    Map width [cells]
    """

    height: uint32 = 0
    """
    Map height [cells]
    """

    origin: Pose = default_field(Pose)
    """
    The origin of the map [m, m, rad].  This is the real-world pose of the
    bottom left corner of cell (0,0) in the map.
    """


@dataclass
class OccupancyGrid(IdlStruct, typename='nav_msgs/OccupancyGrid'):
    """
    This represents a 2-D grid map
    """
    header: Header = default_field(Header)

    info: MapMetaData = default_field(MapMetaData)
    """
    MetaData for the map
    """

    data: sequence[int8] = default_field([])
    """
    The map data, in row-major order, starting with (0,0). 
    Cell (1, 0) will be listed second, representing the next cell in the x direction. 
    Cell (0, 1) will be at the index equal to info.width, followed by (1, 1).
    The values inside are application dependent, but frequently, 
    0 represents unoccupied, 1 represents definitely occupied, and
    -1 represents unknown. 
    """


@dataclass
class Odometry(IdlStruct, typename='nav_msgs/Odometry'):
    """
    This represents an estimate of a position and velocity in free space.
    The pose in this message should be specified in the coordinate frame given by header.frame_id
    The twist in this message should be specified in the coordinate frame given by the child_frame_id
    """
    header: Header = default_field(Header)
    """
    Includes the frame id of the pose parent.
    """

    child_frame_id: str = ''
    """
    Frame id the pose points to. The twist is in this coordinate frame.
    """

    pose: PoseWithCovariance = default_field(PoseWithCovariance)
    """
    Estimated pose that is typically relative to a fixed world frame.
    """

    twist: TwistWithCovariance = default_field(TwistWithCovariance)
    """
    Estimated linear and angular velocity relative to child_frame_id.
    """


@dataclass
class Path(IdlStruct, typename='nav_msgs/Path'):
    """
    An array of poses that represents a Path for a robot to follow.
    """
    header: Header = default_field(Header)
    """
    Indicates the frame_id of the path.
    """
    poses: sequence[PoseStamped] = default_field([])
    """
    Array of poses to follow.
    """


# Schema registry support
_TYPES = {
    "GridCells": GridCells,
    "MapMetaData": MapMetaData,
    "OccupancyGrid": OccupancyGrid,
    "Odometry": Odometry,
    "Path": Path,
}


def is_type_supported(type_name: str) -> bool:
    """Check if a type name is supported by this module."""
    return type_name in _TYPES


def list_types() -> list[str]:
    """List all type schema names in this module."""
    return [f"nav_msgs/msg/{name}" for name in sorted(_TYPES.keys())]


def get_type(type_name: str) -> type:
    """Get the type class by name. Returns None if not found."""
    return _TYPES.get(type_name)



# SPDX-License-Identifier: Apache-2.0
# Copyright © 2025 Au-Zone Technologies. All Rights Reserved.

from dataclasses import dataclass
from enum import Enum

from pycdr2 import IdlStruct
from pycdr2.types import (array, float32, float64, int8, int32, sequence,
                          uint8, uint16, uint32)

from . import default_field
from .builtin_interfaces import Time
from .geometry_msgs import (Point32, Quaternion, Transform, Twist, Vector3,
                            Wrench)
from .std_msgs import Header


class PowerSupplyStatus(Enum):
    """
    Power supply status constants
    """
    POWER_SUPPLY_STATUS_UNKNOWN = 0
    POWER_SUPPLY_STATUS_CHARGING = 1
    POWER_SUPPLY_STATUS_DISCHARGING = 2
    POWER_SUPPLY_STATUS_NOT_CHARGING = 3
    POWER_SUPPLY_STATUS_FULL = 4


class PowerSupplyHealth(Enum):
    """
    Power supply health constants
    """
    POWER_SUPPLY_HEALTH_UNKNOWN = 0
    POWER_SUPPLY_HEALTH_GOOD = 1
    POWER_SUPPLY_HEALTH_OVERHEAT = 2
    POWER_SUPPLY_HEALTH_DEAD = 3
    POWER_SUPPLY_HEALTH_OVERVOLTAGE = 4
    POWER_SUPPLY_HEALTH_UNSPEC_FAILURE = 5
    POWER_SUPPLY_HEALTH_COLD = 6
    POWER_SUPPLY_HEALTH_WATCHDOG_TIMER_EXPIRE = 7
    POWER_SUPPLY_HEALTH_SAFETY_TIMER_EXPIRE = 8


class PowerSupplyTech(Enum):
    """
    Power supply technology (chemistry) constants
    """
    POWER_SUPPLY_TECHNOLOGY_UNKNOWN = 0
    POWER_SUPPLY_TECHNOLOGY_NIMH = 1
    POWER_SUPPLY_TECHNOLOGY_LION = 2
    POWER_SUPPLY_TECHNOLOGY_LIPO = 3
    POWER_SUPPLY_TECHNOLOGY_LIFE = 4
    POWER_SUPPLY_TECHNOLOGY_NICD = 5
    POWER_SUPPLY_TECHNOLOGY_LIMN = 6


@dataclass
class RegionOfInterest(IdlStruct, typename='sensor_msgs/RegionOfInterest'):
    """
    This message is used to specify a region of interest within an image.

    When used to specify the ROI setting of the camera when the image was
    taken, the height and width fields should either match the height and
    width fields for the associated image; or height = width = 0
    indicates that the full resolution image was captured.
    """
    x_offset: uint32 = 0
    """
    Leftmost pixel of the ROI
    (0 if the ROI includes the left edge of the image)
    """
    y_offset: uint32 = 0
    """
    Topmost pixel of the ROI
    (0 if the ROI includes the top edge of the image)
    """
    height: uint32 = 0
    """
    Height of ROI
    """
    width: uint32 = 0
    """
    Width of ROI
    """

    do_rectify: bool = False
    """
    True if a distinct rectified ROI should be calculated from the "raw"
    ROI in this message. Typically this should be False if the full image
    is captured (ROI not used), and True if a subwindow is captured (ROI
    used).
    """


@dataclass
class BatteryState(IdlStruct, typename='sensor_msgs/BatteryState'):
    """
    Constants are chosen to match the enums in the linux kernel
    defined in include/linux/power_supply.h as of version 3.7
    The one difference is for style reasons the constants are
    all uppercase not mixed case.
    """
    header: Header = default_field(Header)
    voltage: float32 = 0
    """
    Voltage in Volts (Mandatory)
    """
    temperature: float32 = 0
    """
    Temperature in Degrees Celsius (If unmeasured NaN)
    """
    current: float32 = 0
    """
    Negative when discharging (A)  (If unmeasured NaN)
    """
    charge: float32 = 0
    """
    Current charge in Ah  (If unmeasured NaN)
    """
    capacity: float32 = 0
    """
    Capacity in Ah (last full capacity)  (If unmeasured NaN)
    """
    design_capacity: float32 = 0
    """
    Capacity in Ah (design capacity)  (If unmeasured NaN)
    """
    percentage: float32 = 0
    """
    Charge percentage on 0 to 1 range  (If unmeasured NaN)
    """
    power_supply_status: uint8 = 0
    """
    The charging status as reported. Values defined above
    """
    power_supply_health: uint8 = 0
    """
    The battery health metric. Values defined above
    """
    power_supply_technology: uint8 = 0
    """
    The battery chemistry. Values defined above
    """
    present: bool = False
    """
    True if the battery is present
    """
    cell_voltage: sequence[float32] = default_field([])
    """
    An array of individual cell voltages for each cell in the pack
    If individual voltages unknown but number of cells known set each to NaN
    """
    cell_temperature: sequence[float32] = default_field([])
    """
    An array of individual cell temperatures for each cell in the pack
    If individual temperatures unknown but number of cells known set each to NaN
    """
    location: str = ''
    """
    The location into which the battery is inserted. (slot number or plug)
    """
    serial_number: str = ''
    """
    The best approximation of the battery serial number
    """


@dataclass
class CameraInfo(IdlStruct, typename='sensor_msgs/CameraInfo'):
    """
    This message defines meta information for a camera. It should be in a
    camera namespace on topic "camera_info" and accompanied by up to five
    image topics named:

    - image_raw: raw data from the camera driver, possibly Bayer encoded
    - image: monochrome, distorted
    - image_color: color, distorted
    - image_rect: monochrome, rectified
    - image_rect_color: color, rectified

    The image_pipeline contains packages (image_proc, stereo_image_proc)
    for producing the four processed image topics from image_raw and
    camera_info. The meaning of the camera parameters are described in
    detail at http://www.ros.org/wiki/image_pipeline/CameraInfo.

    The image_geometry package provides a user-friendly interface to
    common operations using this meta information. If you want to, e.g.,
    project a 3d point into image coordinates, we strongly recommend
    using image_geometry.

    If the camera is uncalibrated, the matrices D, K, R, P should be left
    zeroed out. In particular, clients may assume that K[0] == 0.0
    indicates an uncalibrated camera.
    """
    #######################################################################
    #                     Image acquisition info                          #
    #######################################################################
    header: Header = default_field(Header)
    """
    Time of image acquisition, camera coordinate frame ID
    Header timestamp should be acquisition time of image
    Header frame_id should be optical frame of camera
    origin of frame should be optical center of camera
    +x should point to the right in the image
    +y should point down in the image
    +z should point into the plane of the image
    """

    #######################################################################
    #                      Calibration Parameters                         #
    #######################################################################
    # These are fixed during camera calibration. Their values will be the #
    # same in all messages until the camera is recalibrated. Note that    #
    # self-calibrating systems may "recalibrate" frequently.              #
    #                                                                     #
    # The internal parameters can be used to warp a raw (distorted) image #
    # to:                                                                 #
    #   1. An undistorted image (requires D and K)                        #
    #   2. A rectified image (requires D, K, R)                           #
    # The projection matrix P projects 3D points into the rectified image.#
    #######################################################################

    # The image dimensions with which the camera was calibrated.
    # Normally this will be the full camera resolution in pixels.
    height: uint32 = 0
    width: uint32 = 0

    distortion_model: str = ''
    """
    The distortion model used. Supported models are listed in
    sensor_msgs/distortion_models.hpp. For most cameras, "plumb_bob" - a
    simple model of radial and tangential distortion - is sufficent.
    """

    d: sequence[float64] = default_field([])
    """
    The distortion parameters, size depending on the distortion model.
    For "plumb_bob", the 5 parameters are: (k1, k2, t1, t2, k3).
    """

    k: array[float64, 9] = default_field([0] * 9)
    """
    3x3 row-major matrix
    Intrinsic camera matrix for the raw (distorted) images.
        [fx  0 cx]
    K = [ 0 fy cy]
        [ 0  0  1]
    Projects 3D points in the camera coordinate frame to 2D pixel
    coordinates using the focal lengths (fx, fy) and principal point
    (cx, cy).
    """

    r: array[float64, 9] = default_field([0] * 9)
    """
    3x3 row-major matrix
    Rectification matrix (stereo cameras only)
    A rotation matrix aligning the camera coordinate system to the ideal
    stereo image plane so that epipolar lines in both stereo images are
    parallel.
    """

    p: array[float64, 12] = default_field([0] * 12)
    """
    3x4 row-major matrix
    Projection/camera matrix
        [fx'  0  cx' Tx]
    P = [ 0  fy' cy' Ty]
        [ 0   0   1   0]
    By convention, this matrix specifies the intrinsic (camera) matrix
     of the processed (rectified) image. That is, the left 3x3 portion
     is the normal camera intrinsic matrix for the rectified image.
    It projects 3D points in the camera coordinate frame to 2D pixel
     coordinates using the focal lengths (fx', fy') and principal point
     (cx', cy') - these may differ from the values in K.
    For monocular cameras, Tx = Ty = 0. Normally, monocular cameras will
     also have R = the identity and P[1:3,1:3] = K.
    For a stereo pair, the fourth column [Tx Ty 0]' is related to the
     position of the optical center of the second camera in the first
     camera's frame. We assume Tz = 0 so both cameras are in the same
     stereo image plane. The first camera always has Tx = Ty = 0. For
     the right (second) camera of a horizontal stereo pair, Ty = 0 and
     Tx = -fx' * B, where B is the baseline between the cameras.
    Given a 3D point [X Y Z]', the projection (x, y) of the point onto
     the rectified image is given by:
     [u v w]' = P * [X Y Z 1]'
            x = u / w
            y = v / w
     This holds for both images of a stereo pair.
    """
    #######################################################################
    #                      Operational Parameters                         #
    #######################################################################
    # These define the image region actually captured by the camera       #
    # driver. Although they affect the geometry of the output image, they #
    # may be changed freely without recalibrating the camera.             #
    #######################################################################

    # Binning refers here to any camera setting which combines rectangular
    #  neighborhoods of pixels into larger "super-pixels." It reduces the
    #  resolution of the output image to
    #  (width / binning_x) x (height / binning_y).
    # The default values binning_x = binning_y = 0 is considered the same
    #  as binning_x = binning_y = 1 (no subsampling).
    binning_x: uint32 = 0
    binning_y: uint32 = 0

    roi: RegionOfInterest = default_field(RegionOfInterest)
    """
    Region of interest (subwindow of full camera resolution), given in
     full resolution (unbinned) image coordinates. A particular ROI
     always denotes the same window of pixels on the camera sensor,
     regardless of binning settings.
    The default setting of roi (all values 0) is considered the same as
     full resolution (roi.width = width, roi.height = height).
    """


@dataclass
class ChannelFloat32(IdlStruct, typename='sensor_msgs/ChannelFloat32'):
    """
    This message is used by the PointCloud message to hold optional data
    associated with each point in the cloud. The length of the values
    array should be the same as the length of the points array in the
    PointCloud, and each value should be associated with the corresponding
    point.

    Channel names in existing practice include:
      "u", "v" - row and column (respectively) in the left stereo image.
                 This is opposite to usual conventions but remains for
                 historical reasons. The newer PointCloud2 message has no
                 such problem.
      "rgb" - For point clouds produced by color stereo cameras. uint8
              (R,G,B) values packed into the least significant 24 bits,
              in order.
      "intensity" - laser or pixel intensity.
      "distance"
    """

    name: str = ''
    """
    The channel name should give semantics of the channel (e.g.
    "intensity" instead of "value").
    """

    values: sequence[float32] = default_field([])
    """
    The values array should be 1-1 with the elements of the associated
    PointCloud.
    """


@dataclass
class CompressedImage(IdlStruct, typename='sensor_msgs/CompressedImage'):
    """
    This message contains a compressed image.
    """
    header: Header = default_field(Header)
    """
    Header timestamp should be acquisition time of image
    Header frame_id should be optical frame of camera
    origin of frame should be optical center of cameara
    +x should point to the right in the image
    +y should point down in the image
    +z should point into to plane of the image
    """
    format: str = ''
    """
    Specifies the format of the data
      Acceptable values:
        jpeg, png, tiff
    """
    data: sequence[uint8] = default_field([])
    """
    Compressed image buffer
    """


@dataclass
class FluidPressure(IdlStruct, typename='sensor_msgs/FluidPressure'):
    """
    Single pressure reading.  This message is appropriate for measuring the
    pressure inside of a fluid (air, water, etc).  This also includes
    atmospheric or barometric pressure.

    This message is not appropriate for force/pressure contact sensors.
    """
    header: Header = default_field(Header)
    """
    timestamp of the measurement
    frame_id is the location of the pressure sensor
    """
    fluid_pressure: float64 = 0
    """
    Absolute pressure reading in Pascals.
    """
    variance: float64 = 0
    """
    0 is interpreted as variance unknown
    """


@dataclass
class Illuminance(IdlStruct, typename='sensor_msgs/Illuminance'):
    """
    Single photometric illuminance measurement.  Light should be assumed to be
    measured along the sensor's x-axis (the area of detection is the y-z plane).
    The illuminance should have a 0 or positive value and be received with
    the sensor's +X axis pointing toward the light source.

    Photometric illuminance is the measure of the human eye's sensitivity of the
    intensity of light encountering or passing through a surface.

    All other Photometric and Radiometric measurements should not use this message.
    This message cannot represent:
     - Luminous intensity (candela/light source output)
     - Luminance (nits/light output per area)
     - Irradiance (watt/area), etc.
    """
    header: Header = default_field(Header)
    """
    timestamp is the time the illuminance was measured
    frame_id is the location and direction of the reading
    """
    illuminance: float64 = 0
    """
    Measurement of the Photometric Illuminance in Lux.
    """
    variance: float64 = 0
    """
    0 is interpreted as variance unknown
    """


@dataclass
class Image(IdlStruct, typename='sensor_msgs/Image'):
    """
    This message contains an uncompressed image
    (0, 0) is at top-left corner of image
    """
    header: Header = default_field(Header)
    """
    Header timestamp should be acquisition time of image
    Header frame_id should be optical frame of camera
    origin of frame should be optical center of cameara
    +x should point to the right in the image
    +y should point down in the image
    +z should point into to plane of the image
    If the frame_id here and the frame_id of the CameraInfo
    message associated with the image conflict
    the behavior is undefined
    """
    height: uint32 = 0
    """
    image height, that is, number of rows
    """
    width: uint32 = 0
    """
    image width, that is, number of columns
    """

    encoding: str = ''
    """
    The legal values for encoding are in file src/image_encodings.cpp
    If you want to standardize a new string format, join
    ros-users@lists.ros.org and send an email proposing a new encoding.
    Encoding of pixels -- channel meaning, ordering, size
    taken from the list of strings in include/sensor_msgs/image_encodings.hpp
    """
    is_bigendian: uint8 = 0
    """
    is this data bigendian?
    """
    step: uint32 = 0
    """
    Full row length in bytes
    """
    data: sequence[uint8] = default_field([])
    """
    actual matrix data, size is (step * rows)
    """


@dataclass
class Imu(IdlStruct, typename='sensor_msgs/Imu'):
    """
    This is a message to hold data from an IMU (Inertial Measurement Unit)

    Accelerations should be in m/s^2 (not in g's), and rotational velocity should be in rad/sec

    If the covariance of the measurement is known, it should be filled in (if all you know is the
    variance of each measurement, e.g. from the datasheet, just put those along the diagonal)
    A covariance matrix of all zeros will be interpreted as "covariance unknown", and to use the
    data a covariance will have to be assumed or gotten from some other source

    If you have no estimate for one of the data elements (e.g. your IMU doesn't produce an
    orientation estimate), please set element 0 of the associated covariance matrix to -1
    If you are interpreting this message, please check for a value of -1 in the first element of each
    covariance matrix, and disregard the associated estimate.
    """
    header: Header = default_field(Header)

    orientation: Quaternion = default_field(Quaternion)
    orientation_covariance: array[float64, 9] = default_field([0] * 9)
    """
    Row major about x, y, z axes
    """

    angular_velocity: Vector3 = default_field(Vector3)
    angular_velocity_covariance: array[float64, 9] = default_field([0] * 9)
    """
    Row major about x, y, z axes
    """

    linear_acceleration: Vector3 = default_field(Vector3)
    linear_acceleration_covariance: array[float64, 9] = default_field([0] * 9)
    """
    Row major x, y z
    """


@dataclass
class JointState(IdlStruct, typename='sensor_msgs/JointState'):
    """
    This is a message that holds data to describe the state of a set of torque controlled joints.

    The state of each joint (revolute or prismatic) is defined by:
     * the position of the joint (rad or m),
     * the velocity of the joint (rad/s or m/s) and
     * the effort that is applied in the joint (Nm or N).

    Each joint is uniquely identified by its name
    The header specifies the time at which the joint states were recorded. All the joint states
    in one message have to be recorded at the same time.

    This message consists of a multiple arrays, one for each part of the joint state.
    The goal is to make each of the fields optional. When e.g. your joints have no
    effort associated with them, you can leave the effort array empty.

    All arrays in this message should have the same size, or be empty.
    This is the only way to uniquely associate the joint name with the correct
    states.
    """
    header: Header = default_field(Header)

    name: sequence[str] = default_field([])
    position: sequence[float64] = default_field([])
    velocity: sequence[float64] = default_field([])
    effort: sequence[float64] = default_field([])


@dataclass
class Joy(IdlStruct, typename='sensor_msgs/Joy'):
    """
    Reports the state of a joystick's axes and buttons.
    """
    header: Header = default_field(Header)
    """
    The timestamp is the time at which data is received from the joystick.
    """

    axes: sequence[float32] = default_field([])
    """
    The axes measurements from a joystick.
    """

    buttons: sequence[int32] = default_field([])
    """
    The buttons measurements from a joystick.
    """


class FeedbackType(Enum):
    TYPE_LED = 0
    TYPE_RUMBLE = 1
    TYPE_BUZZER = 2


@dataclass
class JoyFeedback(IdlStruct, typename='sensor_msgs/JoyFeedback'):
    type: uint8 = 0
    """
    Declare of the type of feedback
    """

    id: uint8 = 0
    """
    This will hold an id number for each type of each feedback.
    Example, the first led would be id=0, the second would be id=1
    """

    intensity: float32 = 0
    """
    Intensity of the feedback, from 0.0 to 1.0, inclusive.  If device is
    actually binary, driver should treat 0<=x<0.5 as off, 0.5<=x<=1 as on.
    """


@dataclass
class JoyFeedbackArray(IdlStruct, typename='sensor_msgs/JoyFeedbackArray'):
    """
    This message publishes values for multiple feedback at once.
    """
    array: sequence[JoyFeedback] = default_field([])


@dataclass
class LaserEcho(IdlStruct, typename='sensor_msgs/LaserEcho'):
    """
    This message is a submessage of MultiEchoLaserScan and is not intended
    to be used separately.
    """
    echoes: sequence[float32] = default_field([])
    """
    Multiple values of ranges or intensities.
    Each array represents data from the same angle increment.
    """


@dataclass
class LaserScan(IdlStruct, typename='sensor_msgs/LaserScan'):
    """
    Single scan from a planar laser range-finder

    If you have another ranging device with different behavior (e.g. a sonar
    array), please find or create a different message, since applications
    will make fairly laser-specific assumptions about this data
    """
    header: Header = default_field(Header)
    """
    timestamp in the header is the acquisition time of
    the first ray in the scan.
    
    in frame frame_id, angles are measured around
    the positive Z axis (counterclockwise, if Z is up)
    with zero angle being forward along the x axis
    """
    angle_min: float32 = 0
    """
    start angle of the scan [rad]
    """
    angle_max: float32 = 0
    """
    end angle of the scan [rad]
    """
    angle_increment: float32 = 0
    """
    angular distance between measurements [rad]
    """
    time_increment: float32 = 0
    """
    time between measurements [seconds] - if your scanner
    is moving, this will be used in interpolating position
    of 3d points
    """
    scan_time: float32 = 0
    """
    time between scans [seconds]
    """
    range_min: float32 = 0
    """
    minimum range value [m]
    """
    range_max: float32 = 0
    """
    maximum range value [m]
    """
    ranges: sequence[float32] = default_field([])
    """ 
    range data [m]
    (Note: values < range_min or > range_max should be discarded)
    """
    intensities: sequence[float32] = default_field([])
    """
    intensity data [device-specific units].  If your
    device does not provide intensities, please leave
    the array empty.
    """


@dataclass
class MagneticField(IdlStruct, typename='sensor_msgs/MagneticField'):
    """
    Measurement of the Magnetic Field vector at a specific location.

    If the covariance of the measurement is known, it should be filled in.
    If all you know is the variance of each measurement, e.g. from the datasheet,
    just put those along the diagonal.
    A covariance matrix of all zeros will be interpreted as "covariance unknown",
    and to use the data a covariance will have to be assumed or gotten from some
    other source.
    """
    header: Header = default_field(Header)
    """
    timestamp is the time the
    field was measured
    frame_id is the location and orientation
    of the field measurement
    """
    magnetic_field: Vector3 = default_field(Vector3)
    """
    x, y, and z components of the
    field vector in Tesla
    If your sensor does not output 3 axes,
    put NaNs in the components not reported.
    """
    magnetic_field_covariance: array[float64, 9] = default_field([0] * 9)
    """
    Row major about x, y, z axes
    0 is interpreted as variance unknown
    """


@dataclass
class MultiDOFJointState(IdlStruct, typename='sensor_msgs/MultiDOFJointState'):
    """
    Representation of state for joints with multiple degrees of freedom,
    following the structure of JointState which can only represent a single degree of freedom.

    It is assumed that a joint in a system corresponds to a transform that gets applied
    along the kinematic chain. For example, a planar joint (as in URDF) is 3DOF (x, y, yaw)
    and those 3DOF can be expressed as a transformation matrix, and that transformation
    matrix can be converted back to (x, y, yaw)

    Each joint is uniquely identified by its name
    The header specifies the time at which the joint states were recorded. All the joint states
    in one message have to be recorded at the same time.

    This message consists of a multiple arrays, one for each part of the joint state.
    The goal is to make each of the fields optional. When e.g. your joints have no
    wrench associated with them, you can leave the wrench array empty.

    All arrays in this message should have the same size, or be empty.
    This is the only way to uniquely associate the joint name with the correct
    states.
    """
    header: Header = default_field(Header)

    joint_names: sequence[str] = default_field([])
    transforms: sequence[Transform] = default_field([])
    twist: sequence[Twist] = default_field([])
    wrench: sequence[Wrench] = default_field([])


@dataclass
class MultiEchoLaserScan(IdlStruct, typename='sensor_msgs/MultiEchoLaserScan'):
    """
    Single scan from a multi-echo planar laser range-finder

    If you have another ranging device with different behavior (e.g. a sonar
    array), please find or create a different message, since applications
    will make fairly laser-specific assumptions about this data
    """
    header: Header = default_field(Header)
    """
    timestamp in the header is the acquisition time of
    the first ray in the scan.
    
    in frame frame_id, angles are measured around
    the positive Z axis (counterclockwise, if Z is up)
    with zero angle being forward along the x axis
    """
    angle_min: float32 = 0
    """
    start angle of the scan [rad]
    """
    angle_max: float32 = 0
    """
    end angle of the scan [rad]
    """
    angle_increment: float32 = 0
    """
    angular distance between measurements [rad]
    """
    time_increment: float32 = 0
    """
    time between measurements [seconds] - if your scanner
    is moving, this will be used in interpolating position
    of 3d points
    """
    scan_time: float32 = 0
    """
    time between scans [seconds]
    """
    range_min: float32 = 0
    """
    minimum range value [m]
    """
    range_max: float32 = 0
    """
    maximum range value [m]
    """
    ranges: sequence[LaserEcho] = default_field([])
    """
    range data [m]
    (Note: NaNs, values < range_min or > range_max should be discarded)
    +Inf measurements are out of range
    -Inf measurements are too close to determine exact distance.
    """
    intensities: sequence[LaserEcho] = default_field([])
    """
    intensity data [device-specific units].  If your
    device does not provide intensities, please leave
    the array empty.
    """


class GpsStatus(Enum):
    STATUS_NO_FIX = -1        # unable to fix position
    STATUS_FIX = 0        # unaugmented fix
    STATUS_SBAS_FIX = 1        # with satellite-based augmentation
    STATUS_GBAS_FIX = 2        # with ground-based augmentation


class GpsService(Enum):
    SERVICE_GPS = 1
    SERVICE_GLONASS = 2
    SERVICE_COMPASS = 4      # includes BeiDou.
    SERVICE_GALILEO = 8


@dataclass
class NavSatStatus(IdlStruct, typename='sensor_msgs/NavSatStatus'):
    """
    Navigation Satellite fix status for any Global Navigation Satellite System.

    Whether to output an augmented fix is determined by both the fix
    type and the last time differential corrections were received.  A
    fix is valid when status >= STATUS_FIX.
    """
    status: int8 = 0
    """
    Bits defining which Global Navigation Satellite System signals were
    used by the receiver.
    """
    service: uint16 = 0


class CovarianceType(Enum):
    COVARIANCE_TYPE_UNKNOWN = 0
    COVARIANCE_TYPE_APPROXIMATED = 1
    COVARIANCE_TYPE_DIAGONAL_KNOWN = 2
    COVARIANCE_TYPE_KNOWN = 3


@dataclass
class NavSatFix(IdlStruct, typename='sensor_msgs/NavSatFix'):
    """
    Navigation Satellite fix for any Global Navigation Satellite System

    Specified using the WGS 84 reference ellipsoid
    """

    header: Header = default_field(Header)
    """
    header.stamp specifies the ROS time for this measurement (the
           corresponding satellite time may be reported using the
           sensor_msgs/TimeReference message).
    
    header.frame_id is the frame of reference reported by the satellite
           receiver, usually the location of the antenna.  This is a
           Euclidean frame relative to the vehicle, not a reference
           ellipsoid.
    """

    status: NavSatStatus = default_field(NavSatStatus)
    """
    Satellite fix status information.
    """

    latitude: float64 = 0
    """
    Latitude [degrees]. Positive is north of equator; negative is south.
    """

    longitude: float64 = 0
    """
    Longitude [degrees]. Positive is east of prime meridian; negative is west.
    """

    altitude: float64 = 0
    """
    Altitude [m]. Positive is above the WGS 84 ellipsoid
    (quiet NaN if no altitude is available).
    """

    position_covariance: array[float64, 9] = default_field([0] * 9)
    """
    Position covariance [m^2] defined relative to a tangential plane
    through the reported position. The components are East, North, and
    Up (ENU), in row-major order.
    
    Beware: this coordinate system exhibits singularities at the poles.
    """

    position_covariance_type: uint8 = 0
    """
    If the covariance of the fix is known, fill it in completely. If the
    GPS receiver provides the variance of each measurement, put them
    along the diagonal. If only Dilution of Precision is available,
    estimate an approximate covariance from that.
    """


@dataclass
class PointCloud(IdlStruct, typename='sensor_msgs/PointCloud'):
    """
    # THIS MESSAGE IS DEPRECATED AS OF FOXY
    # Please use sensor_msgs/PointCloud2

    This message holds a collection of 3d points, plus optional additional
    information about each point.
    """
    header: Header = default_field(Header)
    """
    Time of sensor data acquisition, coordinate frame ID.
    """

    points: sequence[Point32] = default_field([])
    """
    Array of 3d points. Each Point32 should be interpreted as a 3d point
    in the frame given in the header.
    """

    channels: sequence[ChannelFloat32] = default_field([])
    """
    Each channel should have the same number of elements as points array,
    and the data in each channel should correspond 1:1 with each point.
    Channel names in common practice are listed in ChannelFloat32.msg.
    """


class PointFieldDatatype(Enum):
    INT8 = 1
    UINT8 = 2
    INT16 = 3
    UINT16 = 4
    INT32 = 5
    UINT32 = 6
    FLOAT32 = 7
    FLOAT64 = 8


@dataclass
class PointField(IdlStruct, typename='sensor_msgs/PointField'):
    """
    This message holds the description of one point entry in the
    PointCloud2 message format.
    """
    name: str = ''
    """
    Name of field
    Common PointField names are x, y, z, intensity, rgb, rgba
    """
    offset: uint32 = 0
    """
    Offset from start of point struct
    """
    datatype: uint8 = 0
    """
    Datatype enumeration, see above
    """
    count: uint32 = 0
    """
    How many elements in the field
    """


@dataclass
class PointCloud2(IdlStruct, typename='sensor_msgs/PointCloud2'):
    """
    This message holds a collection of N-dimensional points, which may
    contain additional information such as normals, intensity, etc. The
    point data is stored as a binary blob, its layout described by the
    contents of the "fields" array.

    The point cloud data may be organized 2d (image-like) or 1d (unordered).
    Point clouds organized as 2d images may be produced by camera depth sensors
    such as stereo or time-of-flight.
    """

    header: Header = default_field(Header)
    """
    Time of sensor data acquisition, and the coordinate frame ID (for 3d points).
    """

    # 2D structure of the point cloud. If the cloud is unordered, height is
    # 1 and width is the length of the point cloud.
    height: uint32 = 0
    width: uint32 = 0

    fields: sequence[PointField] = default_field([])
    """
    Describes the channels and their layout in the binary data blob.
    """

    is_bigendian: bool = False
    """
    Is this data bigendian?
    """
    point_step: uint32 = 0
    """
    Length of a point in bytes
    """
    row_step: uint32 = 0
    """
    Length of a row in bytes
    """
    data: sequence[uint8] = default_field([])
    """
    Actual point data, size is (row_step*height)
    """
    is_dense: bool = False
    """
    True if there are no invalid points
    """


class RadiationType(Enum):
    """
    Radiation type enums
    If you want a value added to this list, send an email to the ros-users list
    """
    ULTRASOUND = 0
    INFRARED = 1


@dataclass
class Range(IdlStruct, typename='sensor_msgs/Range'):
    """
    Single range reading from an active ranger that emits energy and reports
    one range reading that is valid along an arc at the distance measured.
    This message is  not appropriate for laser scanners. See the LaserScan
    message if you are working with a laser scanner.

    This message also can represent a fixed-distance (binary) ranger.  This
    sensor will have min_range===max_range===distance of detection.
    These sensors follow REP 117 and will output -Inf if the object is detected
    and +Inf if the object is outside of the detection range.
    """
    header: Header = default_field(Header)
    """
    timestamp in the header is the time the ranger
    returned the distance reading
    """

    radiation_type: uint8 = 0
    """
    the type of radiation used by the sensor
    (sound, IR, etc) [enum]
    """
    field_of_view: float32 = 0
    """
    the size of the arc that the distance reading is
    valid for [rad]
    the object causing the range reading may have
    been anywhere within -field_of_view/2 and
    field_of_view/2 at the measured range.
    0 angle corresponds to the x-axis of the sensor.
    """

    min_range: float32 = 0
    """
    minimum range value [m]
    """
    max_range: float32 = 0
    """
    maximum range value [m]
    Fixed distance rangers require min_range==max_range
    """
    range: float32 = 0
    """ 
    range data [m]
    (Note: values < range_min or > range_max should be discarded)
    Fixed distance rangers only output -Inf or +Inf.
    -Inf represents a detection within fixed distance.
    (Detection too close to the sensor to quantify)
    +Inf represents no detection within the fixed distance.
    (Object out of range)
    """


@dataclass
class RelativeHumidity(IdlStruct, typename='sensor_msgs/RelativeHumidity'):
    """
    Single reading from a relative humidity sensor.
    Defines the ratio of partial pressure of water vapor to the saturated vapor
    pressure at a temperature.
    """
    header: Header = default_field(Header)
    """
    timestamp of the measurement
    frame_id is the location of the humidity sensor
    """
    relative_humidity: float64 = 0
    """
    Expression of the relative humidity
    from 0.0 to 1.0.
    0.0 is no partial pressure of water vapor
    1.0 represents partial pressure of saturation
    """
    variance: float64 = 0
    """
    0 is interpreted as variance unknown
    """


@dataclass
class Temperature(IdlStruct, typename='sensor_msgs/Temperature'):
    """
    Single temperature reading.
    """
    header: Header = default_field(Header)
    """
    timestamp is the time the temperature was measured
    frame_id is the location of the temperature reading
    """
    temperature: float64 = 0
    """
    Measurement of the Temperature in Degrees Celsius.
    """
    variance: float64 = 0
    """
    0 is interpreted as variance unknown.
    """


@dataclass
class TimeReference(IdlStruct, typename='sensor_msgs/TimeReference'):
    """
    Measurement from an external time source not actively synchronized with the system clock.
    """
    header: Header = default_field(Header)
    """
    stamp is system time for which measurement was valid
    frame_id is not used
    """
    time_ref: Time = default_field(Time)
    """
    corresponding time from this external source
    """
    source: str = ''
    """
    (optional) name of time source
    """


# Schema registry support
_TYPES = {
    "BatteryState": BatteryState,
    "CameraInfo": CameraInfo,
    "ChannelFloat32": ChannelFloat32,
    "CompressedImage": CompressedImage,
    "FluidPressure": FluidPressure,
    "Illuminance": Illuminance,
    "Image": Image,
    "Imu": Imu,
    "JointState": JointState,
    "Joy": Joy,
    "JoyFeedback": JoyFeedback,
    "JoyFeedbackArray": JoyFeedbackArray,
    "LaserEcho": LaserEcho,
    "LaserScan": LaserScan,
    "MagneticField": MagneticField,
    "MultiDOFJointState": MultiDOFJointState,
    "MultiEchoLaserScan": MultiEchoLaserScan,
    "NavSatFix": NavSatFix,
    "NavSatStatus": NavSatStatus,
    "PointCloud": PointCloud,
    "PointCloud2": PointCloud2,
    "PointField": PointField,
    "Range": Range,
    "RegionOfInterest": RegionOfInterest,
    "RelativeHumidity": RelativeHumidity,
    "Temperature": Temperature,
    "TimeReference": TimeReference,
}


def is_type_supported(type_name: str) -> bool:
    """Check if a type name is supported by this module."""
    return type_name in _TYPES


def list_types() -> list[str]:
    """List all type schema names in this module."""
    return [f"sensor_msgs/msg/{name}" for name in sorted(_TYPES.keys())]


def get_type(type_name: str) -> type:
    """Get the type class by name. Returns None if not found."""
    return _TYPES.get(type_name)



# SPDX-License-Identifier: Apache-2.0
# Copyright © 2025 Au-Zone Technologies. All Rights Reserved.

from dataclasses import dataclass
from enum import Enum

from pycdr2 import IdlStruct
from pycdr2.types import (float32, int16, int32, sequence, uint8, uint16,
                          uint32, uint64)

from . import default_field
from .builtin_interfaces import Duration, Time
from .std_msgs import Header

class model_info(Enum):
    RAW = 0
    INT8 = 1
    UINT8 = 2
    INT16 = 3
    UINT16 = 4
    FLOAT16 = 5
    INT32 = 6
    UINT32 = 7
    FLOAT32 = 8
    INT64 = 9
    UINT64 = 10
    FLOAT64 = 11
    STRING = 12

@dataclass
class Date(IdlStruct, typename='edgefirst_msgs/Date'):
    """
    The Date type holds the year, month, and day of the month.  It is used in
    the LocalTime message to represent the date of the local time.
    """
    year: uint16 = 0
    month: uint8 = 0
    day: uint8 = 0


@dataclass
class LocalTime(IdlStruct, typename='edgefirst_msgs/LocalTime'):
    """
    The local time interface publishes the current time on the device.  It is
    mainly intended to allow synchronization of multiple MCAP files by the
    EdgeFirst Publisher.  The idea is to calculate the offset from the
    timestamp in the header with the actual local time, then when multiple MCAP
    files have the local time topic recorded the relative offsets can then be
    calculated.
    """

    header: Header = default_field(Header)
    """Message header containing the timestamp and frame id."""

    date: Date = default_field(Date)
    """
    The base date from which the local time is calculated.  This could be an
    epoch such as the standard UNIX 1 January 1970 or it could be the current
    date.  To calculate the real local time both the date, time, and timezone
    are combined into a valid date and time.
    """

    time: Time = default_field(Time)
    """
    The time offset from the date.  If the date is the current day then the
    time is the normal time of day.  If the date is an epoch than many days
    will be represented in the time.
    """

    timezone: int16 = 0
    """
    The timezone offset in minutes from UTC of the time value.  The timezone
    would be +/- 720 (+/- 12 hours).  Minutes are used to allow for partial
    offsets such as Newfoundland in Canada which is UTC-210 (UTC-3h30).
    """


@dataclass
class Track(IdlStruct, typename='edgefirst_msgs/Track'):
    id: str = ''
    """
    Unique identifier for the object track, empty if the object is not tracked.
    """
    lifetime: int32 = 0
    """
    Number of consecutive frames the object has been tracked
    """
    created: Time = default_field(Time)
    """
    Time the track was first added
    """


@dataclass
class Box(IdlStruct, typename='edgefirst_msgs/Box'):
    center_x: float32 = 0
    """
    Normalized x-coordinate of the center
    """
    center_y: float32 = 0
    """
    Normalized y-coordinate of the center
    """
    width: float32 = 0
    """
    Normalized width of the box
    """
    height: float32 = 0
    """
    Normalized height of the box
    """
    label: str = ''
    """
    object label
    """
    score: float32 = 0
    """
    confidence score for detection
    """
    distance: float32 = 0
    """
    Distance of object (if known)
    """
    speed: float32 = 0
    """
    Speed of object (if known)
    """
    track: Track = default_field(Track)
    """
    object tracking, each track includes ID and lifetime information
    """


@dataclass
class Mask(IdlStruct, typename='edgefirst_msgs/Mask'):
    height: uint32 = 0
    """
    The height of the mask, 0 if this dimension is unused.
    """

    width: uint32 = 0
    """
    The width of the mask, 0 if this dimension is unused.
    """

    length: uint32 = 0
    """
    The length of the mask, 0 if this dimension is unused.  The length would
    be used in 3D masks to represent the depth.  It could also be used for 2D
    bird's eye view masks along with width instead of height (elevation).
    """

    encoding: str = ''
    """
    The optional encoding for the mask (currently unused).
    """

    mask: sequence[uint8] = default_field([])
    """
    The segmentation mask data.  The array should be reshaped according to the
    height, width, and length dimensions.  The dimension order is row-major.
    """

    boxed: bool = False
    """
    If this mask is associated to a corresponding box
    """


@dataclass
class Model(IdlStruct, typename='edgefirst_msgs/Model'):
    header: Header = default_field(Header)
    """
    Metadata including timestamp and coordinate frame
    """

    input_time: Duration = default_field(Duration)
    """
    Duration to load inputs into the model
    """

    model_time: Duration = default_field(Duration)
    """
    Duration to run the model, not including input/output/decoding
    """

    output_time: Duration = default_field(Duration)
    """
    Duration to read outputs from the model
    """

    decode_time: Duration = default_field(Duration)
    """
    Duration to decode the outputs from the model, including nms and tracking.
    """

    boxes: sequence[Box] = default_field([])
    """
    Array of detected object bounding boxes.
    """

    mask: sequence[Mask] = default_field([])
    """
    Segmentation masks from the model.  Empty array if model does not generate
    masks.  Generally models will only generate a single mask if they do.
    """

@dataclass
class ModelInfo(IdlStruct, typename='edgefirst_msgs/ModelInfo'):
    header: Header = default_field(Header)
    """
    Metadata including timestamp and coordinate frame
    """
    input_shape: sequence[uint32] = default_field([])
    """
    Shape of the input tensor(s) in the format "height,width,channels" or "height,width,depth,channels"
    """
    input_type: uint8 = model_info.RAW
    """
    Data type of the input tensor(s) (e.g., "float32", "uint8")
    """
    output_shape: sequence[uint32] = default_field([])
    """
    Shape of the output tensor(s) in the format "height,width,channels" or "height,width,depth,channels"
    """
    output_type: uint8 = model_info.RAW
    """
    Data type of the output tensor(s) (e.g., "float32", "uint8")
    """
    labels: sequence[str] = default_field([])
    """
    Array of strings representing the labels used by the model, empty if no labels available
    """
    model_type: str = ''
    """
    Model tasks/types (e.g., ["object_detection", "classification"])
    """
    model_format: str = ''
    """
    Format of the model (e.g., "DeepViewRT", "HailoRT", "RKNN", "TensorRT", "TFLite")
    """
    model_name: str = ''
    """
    Name of the model (if available), otherwise use filename without extension or path
    """

@dataclass
class Detect(IdlStruct, typename='Detect'):
    header: Header = default_field(Header)
    """
    Metadata including timestamp and coordinate frame
    """
    input_timestamp: Time = default_field(Time)
    """
    Timestamp of the input data (e.g., from camera)
    """
    model_time: Time = default_field(Time)
    """
    Timestamp when the object was processed by the model
    """
    output_time: Time = default_field(Time)
    """
    Timestamp when the processed output was available
    """
    boxes: sequence[Box] = default_field([])
    """
    Array of detected object bounding boxes
    """


@dataclass
class DmaBuffer(IdlStruct, typename='DmaBuffer'):
    header: Header = default_field(Header)
    """
    Metadata including timestamp and coordinate frame
    """
    pid: uint32 = 0
    """
    The process id of the service that created the DMA buffer
    """
    fd: int32 = 0
    """
    The file descriptor of the DMA buffer
    """
    width: uint32 = 0
    """
    The width of the image in pixels
    """
    height: uint32 = 0
    """
    The height of the image in pixels
    """
    stride: uint32 = 0
    """
    The stride of the image in bytes
    """
    fourcc: uint32 = 0
    """
    The fourcc code of the image
    """
    length: uint32 = 0
    """
    The length of the DMA buffer in bytes, used to mmap the buffer
    """


class RadarChannel(Enum):
    UNDEFINED = 0
    RANGE = 1
    DOPPLER = 2
    AZIMUTH = 3
    ELEVATION = 4
    RXCHANNEL = 5
    SEQUENCE = 6


@dataclass
class RadarCube(IdlStruct, typename='edgefirst_msgs/RadarCube'):
    """
    The RadarCube interface carries various radar cube reprensentations of the
    Radar FFT before generally being processed by CFAR into a point cloud.  The
    cube coud be R, RD, RAD, RA, and so on where R=Range, D=Dopper, and
    A=Azimuth.

    Dimensional labels are used to describe the radar cube layout.  Not all
    cubes include every label.  Undefined is used for dimensions not covered by
    this list.
    """

    header: Header = default_field(Header)
    """Message header containing the timestamp and frame id."""

    timestamp: uint64 = 0
    """Radar frame timestamp generated on the radar module"""

    layout: sequence[uint8] = default_field([])
    """Radar cube layout provides labels for each dimensions"""

    shape: sequence[uint16] = default_field([])
    """Radar cube shape provides the shape of each dimensions"""

    scales: sequence[float32] = default_field([])
    """
    The scaling factors for the dimensions representing bins.  For dimensions
    taken "as-is" the scale will be 1.0.
    """

    cube: sequence[int16] = default_field([])
    """
    The radar cube data as 16bit integers.  If the is_complex is true then each
    element will be pairs of integers with the first being real and the second
    being imaginary.
    """

    is_complex: bool = False
    """
    True if the radar cube is complex in which case the final dimension will be
    doubled in size to account for the pair of int16 elements representing
    [real,imaginary].
    """


@dataclass
class RadarInfo(IdlStruct, typename='edgefirst_msgs/RadarInfo'):
    """
    The RadarInfo interface carries the current radar configuration and status.
    """

    header: Header = default_field(Header)
    """Message header containing the timestamp and frame id."""

    center_frequency: str = ''
    """Radar center frequency band."""

    frequency_sweep: str = ''
    """The frequency sweep controls the detection range of the radar."""

    range_toggle: str = ''
    """
    The range-toggle mode allows the radar to alternate between various
    frequency sweep configurations.  Applications must handle range toggling as
    targets are not consistent between messages as the frequency alternates.
    """

    detection_sensitivity: str = ''
    """
    The detection sensitivity controls the sensitivity to recognize a target.
    """

    cube: bool = False
    """True if the radar is configured to output radar cubes."""


# Schema registry support
_TYPES = {
    "Box": Box,
    "Date": Date,
    "Detect": Detect,
    "DmaBuffer": DmaBuffer,
    "LocalTime": LocalTime,
    "Mask": Mask,
    "Model": Model,
    "ModelInfo": ModelInfo,
    "RadarCube": RadarCube,
    "RadarInfo": RadarInfo,
    "Track": Track,
}


def is_type_supported(type_name: str) -> bool:
    """Check if a type name is supported by this module."""
    return type_name in _TYPES


def list_types() -> list[str]:
    """List all type schema names in this module."""
    return [f"edgefirst_msgs/msg/{name}" for name in sorted(_TYPES.keys())]


def get_type(type_name: str) -> type:
    """Get the type class by name. Returns None if not found."""
    return _TYPES.get(type_name)



# SPDX-License-Identifier: Apache-2.0
# Copyright © 2025 Au-Zone Technologies. All Rights Reserved.

from dataclasses import dataclass

from pycdr2 import IdlStruct
from pycdr2.types import (bounded_str, float32, float64, int8, int16, int32,
                          int64, sequence, uint8, uint16, uint32, uint64)

from . import default_field
from .builtin_interfaces import Time


@dataclass
class MultiArrayDimension(IdlStruct, typename='std_msgs/MultiArrayDimension'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.
    """
    label: str = ''
    """
    label of given dimension
    """
    size: uint32 = 0
    """
    size of given dimension (in type units)
    """
    stride: uint32 = 0
    """
    stride of given dimension
    """


@dataclass
class MultiArrayLayout(IdlStruct, typename='std_msgs/MultiArrayLayout'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.

    The multiarray declares a generic multi-dimensional array of a
    particular data type.  Dimensions are ordered from outer most
    to inner most.

    Accessors should ALWAYS be written in terms of dimension stride
    and specified outer-most dimension first.

    multiarray(i,j,k) = data[data_offset + dim_stride[1]*i + dim_stride[2]*j + k]

    A standard, 3-channel 640x480 image with interleaved color channels
    would be specified as:

    dim[0].label  = "height"
    dim[0].size   = 480
    dim[0].stride = 3*640*480 = 921600  (note dim[0] stride is just size of image)
    dim[1].label  = "width"
    dim[1].size   = 640
    dim[1].stride = 3*640 = 1920
    dim[2].label  = "channel"
    dim[2].size   = 3
    dim[2].stride = 3

    multiarray(i,j,k) refers to the ith row, jth column, and kth channel.
    """
    dim: sequence[MultiArrayDimension] = default_field([])
    """
    Array of dimension properties
    """
    data_offset: uint32 = 0
    """
    padding bytes at front of data
    """


@dataclass
class Bool(IdlStruct, typename='std_msgs/Bool'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.
    """
    data: bool = False


@dataclass
class Byte(IdlStruct, typename='std_msgs/Byte'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.
    """
    data: bytes = None


@dataclass
class ByteMultiArray(IdlStruct, typename='std_msgs/ByteMultiArray'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.

    Please look at the MultiArrayLayout message definition for
    documentation on all multiarrays.
    """
    layout: MultiArrayLayout = default_field(MultiArrayLayout)
    """
    specification of data layout
    """
    data: sequence[bytes] = default_field([])
    """
    array of data
    """


@dataclass
class Char(IdlStruct, typename='std_msgs/Char'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.
    """
    data: bounded_str[1] = ''


@dataclass
class ColorRGBA(IdlStruct, typename='std_msgs/ColorRGBA'):
    r: float32 = 0
    g: float32 = 0
    b: float32 = 0
    a: float32 = 0


@dataclass
class Float32(IdlStruct, typename='std_msgs/Float32'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.
    """
    data: float32 = 0


@dataclass
class Float32MultiArray(IdlStruct, typename='std_msgs/Float32MultiArray'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.

    Please look at the MultiArrayLayout message definition for
    documentation on all multiarrays.
    """
    layout: MultiArrayLayout = default_field(MultiArrayLayout)
    """
    specification of data layout
    """
    data: sequence[float32] = default_field([])
    """
    array of data
    """


@dataclass
class Float64(IdlStruct, typename='std_msgs/Float64'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.
    """
    data: float64 = 0


@dataclass
class Float64MultiArray(IdlStruct, typename='std_msgs/Float64MultiArray'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.

    Please look at the MultiArrayLayout message definition for
    documentation on all multiarrays.
    """
    layout: MultiArrayLayout = default_field(MultiArrayLayout)
    """
    specification of data layout
    """
    data: sequence[float64] = default_field([])
    """
    array of data
    """


@dataclass
class Header(IdlStruct, typename='std_msgs/Header'):
    """
    Standard metadata for higher-level stamped data types.
    This is generally used to communicate timestamped data
    in a particular coordinate frame.
    """
    stamp: Time = default_field(Time)
    """
    Two-integer timestamp that is expressed as seconds and nanoseconds.
    """
    frame_id: str = ''
    """
    Transform frame with which this data is associated.
    """


@dataclass
class Int8(IdlStruct, typename='std_msgs/Int8'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.
    """
    data: int8 = 0


@dataclass
class Int8MultiArray(IdlStruct, typename='std_msgs/Int8MultiArray'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.

    Please look at the MultiArrayLayout message definition for
    documentation on all multiarrays.
    """
    layout: MultiArrayLayout = default_field(MultiArrayLayout)
    """
    specification of data layout
    """
    data: sequence[int8] = default_field([])
    """
    array of data
    """


@dataclass
class Int16(IdlStruct, typename='std_msgs/Int16'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.
    """
    data: int16 = 0


@dataclass
class Int16MultiArray(IdlStruct, typename='std_msgs/Int16MultiArray'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.

    Please look at the MultiArrayLayout message definition for
    documentation on all multiarrays.
    """
    layout: MultiArrayLayout = default_field(MultiArrayLayout)
    """
    specification of data layout
    """
    data: sequence[int16] = default_field([])
    """
    array of data
    """


@dataclass
class Int32(IdlStruct, typename='std_msgs/Int32'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.
    """
    data: int32 = 0


@dataclass
class Int32MultiArray(IdlStruct, typename='std_msgs/Int32MultiArray'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.

    Please look at the MultiArrayLayout message definition for
    documentation on all multiarrays.
    """
    layout: MultiArrayLayout = default_field(MultiArrayLayout)
    """
    specification of data layout
    """
    data: sequence[int32] = default_field([])
    """
    array of data
    """


@dataclass
class Int64(IdlStruct, typename='std_msgs/Int64'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.
    """
    data: int64 = 0


@dataclass
class Int64MultiArray(IdlStruct, typename='std_msgs/Int64MultiArray'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.

    Please look at the MultiArrayLayout message definition for
    documentation on all multiarrays.
    """
    layout: MultiArrayLayout = default_field(MultiArrayLayout)
    """
    specification of data layout
    """
    data: sequence[int64] = default_field([])
    """
    array of data
    """


@dataclass
class String(IdlStruct, typename='std_msgs/String'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.
    """
    data: str = ''


@dataclass
class Uint8(IdlStruct, typename='std_msgs/Uint8'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.
    """
    data: uint8 = 0


@dataclass
class Uint8MultiArray(IdlStruct, typename='std_msgs/Uint8MultiArray'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.

    Please look at the MultiArrayLayout message definition for
    documentation on all multiarrays.
    """
    layout: MultiArrayLayout = default_field(MultiArrayLayout)
    """
    specification of data layout
    """
    data: sequence[uint8] = default_field([])
    """
    array of data
    """


@dataclass
class Uint16(IdlStruct, typename='std_msgs/Uint16'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.
    """
    data: uint16 = 0


@dataclass
class Uint16MultiArray(IdlStruct, typename='std_msgs/Uint16MultiArray'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.

    Please look at the MultiArrayLayout message definition for
    documentation on all multiarrays.
    """
    layout: MultiArrayLayout = default_field(MultiArrayLayout)
    """
    specification of data layout
    """
    data: sequence[uint16] = default_field([])
    """
    array of data
    """


@dataclass
class Uint32(IdlStruct, typename='std_msgs/Uint32'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.
    """
    data: uint32 = 0


@dataclass
class Uint32MultiArray(IdlStruct, typename='std_msgs/Uint32MultiArray'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.

    Please look at the MultiArrayLayout message definition for
    documentation on all multiarrays.
    """
    layout: MultiArrayLayout = default_field(MultiArrayLayout)
    """
    specification of data layout
    """
    data: sequence[uint32] = default_field([])
    """
    array of data
    """


@dataclass
class Uint64(IdlStruct, typename='std_msgs/Uint64'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.
    """
    data: uint64 = 0


@dataclass
class Uint64MultiArray(IdlStruct, typename='std_msgs/Uint64MultiArray'):
    """
    This was originally provided as an example message.
    It is deprecated as of Foxy
    It is recommended to create your own semantically meaningful message.
    However if you would like to continue using this please use the equivalent in example_msgs.

    Please look at the MultiArrayLayout message definition for
    documentation on all multiarrays.
    """
    layout: MultiArrayLayout = default_field(MultiArrayLayout)
    """
    specification of data layout
    """
    data: sequence[uint64] = default_field([])
    """
    array of data
    """


# Schema registry support
_TYPES = {
    "Bool": Bool,
    "Byte": Byte,
    "ByteMultiArray": ByteMultiArray,
    "Char": Char,
    "ColorRGBA": ColorRGBA,
    "Float32": Float32,
    "Float32MultiArray": Float32MultiArray,
    "Float64": Float64,
    "Float64MultiArray": Float64MultiArray,
    "Header": Header,
    "Int8": Int8,
    "Int8MultiArray": Int8MultiArray,
    "Int16": Int16,
    "Int16MultiArray": Int16MultiArray,
    "Int32": Int32,
    "Int32MultiArray": Int32MultiArray,
    "Int64": Int64,
    "Int64MultiArray": Int64MultiArray,
    "MultiArrayDimension": MultiArrayDimension,
    "MultiArrayLayout": MultiArrayLayout,
    "String": String,
    "Uint8": Uint8,
    "Uint8MultiArray": Uint8MultiArray,
    "Uint16": Uint16,
    "Uint16MultiArray": Uint16MultiArray,
    "Uint32": Uint32,
    "Uint32MultiArray": Uint32MultiArray,
    "Uint64": Uint64,
    "Uint64MultiArray": Uint64MultiArray,
}


def is_type_supported(type_name: str) -> bool:
    """Check if a type name is supported by this module."""
    return type_name in _TYPES


def list_types() -> list[str]:
    """List all type schema names in this module."""
    return [f"std_msgs/msg/{name}" for name in sorted(_TYPES.keys())]


def get_type(type_name: str) -> type:
    """Get the type class by name. Returns None if not found."""
    return _TYPES.get(type_name)



# SPDX-License-Identifier: Apache-2.0
# Copyright © 2025 Au-Zone Technologies. All Rights Reserved.

from dataclasses import dataclass

from pycdr2 import IdlStruct
from pycdr2.types import int32, uint32


@dataclass
class Duration(IdlStruct, typename='builtin_interfaces/Duration'):
    """
    Duration defines a period between two time points.
    Messages of this datatype are of ROS Time following this design:
    https://design.ros2.org/articles/clock_and_time.html
    """
    sec: int32 = 0
    """
    The seconds component, valid over all int32 values.
    """
    nanosec: uint32 = 0
    """
    The nanoseconds component, valid in the range [0, 10e9).
    """


@dataclass
class Time(IdlStruct, typename='builtin_interfaces/Time'):
    sec: int32 = 0
    """
    The seconds component, valid over all int32 values.
    """
    nanosec: uint32 = 0
    """
    The nanoseconds component, valid in the range [0, 10e9).
    """


# Schema registry support
_TYPES = {
    "Duration": Duration,
    "Time": Time,
}


def is_type_supported(type_name: str) -> bool:
    """Check if a type name is supported by this module."""
    return type_name in _TYPES


def list_types() -> list[str]:
    """List all type schema names in this module."""
    return [
        "builtin_interfaces/msg/Duration",
        "builtin_interfaces/msg/Time",
    ]


def get_type(type_name: str) -> type:
    """Get the type class by name. Returns None if not found."""
    return _TYPES.get(type_name)



# SPDX-License-Identifier: Apache-2.0
# Copyright © 2025 Au-Zone Technologies. All Rights Reserved.

from dataclasses import dataclass

from pycdr2 import IdlStruct
from pycdr2.types import array, float32, float64, sequence

from . import default_field
from .std_msgs import Header


@dataclass
class Quaternion(IdlStruct, typename='geometry_msgs/Quaternion'):
    """
    This represents an orientation in free space in quaternion form.
    """
    x: float64 = 0
    y: float64 = 0
    z: float64 = 0
    w: float64 = 1


@dataclass
class QuaternionStamped(IdlStruct, typename='geometry_msgs/QuaternionStamped'):
    """
    This represents an orientation with reference coordinate frame and timestamp.
    """
    header: Header = default_field(Header)
    quaternion: Quaternion = default_field(Quaternion)


@dataclass
class Vector3(IdlStruct, typename='geometry_msgs/Vector3'):
    """
    This represents a vector in free space.
    This is semantically different than a point.
    A vector is always anchored at the origin.
    When a transform is applied to a vector, only the rotational component is applied.
    """
    x: float64 = 0
    y: float64 = 0
    z: float64 = 0


@dataclass
class Vector3Stamped(IdlStruct, typename='geometry_msgs/Vector3Stamped'):
    """
    This represents a Vector3 with reference coordinate frame and timestamp
    Note that this follows vector semantics with it always anchored at the origin,
    so the rotational elements of a transform are the only parts applied when transforming.
    """
    header: Header = default_field(Header)
    vector: Vector3 = default_field(Vector3)


@dataclass
class Accel(IdlStruct, typename='geometry_msgs/Accel'):
    """
    This expresses acceleration in free space broken into its linear and angular parts.
    """
    linear: Vector3 = default_field(Vector3)
    angular: Vector3 = default_field(Vector3)


@dataclass
class AccelStamped(IdlStruct, typename='geometry_msgs/AccelStamped'):
    """
    An accel with reference coordinate frame and timestamp
    """
    header: Header = default_field(Header)
    accel: Accel = default_field(Accel)


@dataclass
class AccelWithCovariance(IdlStruct, typename='geometry_msgs/AccelWithCovariance'):
    """
    This expresses acceleration in free space with uncertainty.
    """
    accel: Accel = default_field(Accel)

    covariance: array[float64, 36] = default_field([0] * 36)
    """
    Row-major representation of the 6x6 covariance matrix
    The orientation parameters use a fixed-axis representation.
    In order, the parameters are:
    (x, y, z, rotation about X axis, rotation about Y axis, rotation about Z axis)
    """


@dataclass
class AccelWithCovarianceStamped(IdlStruct, typename='geometry_msgs/AccelWithCovarianceStamped'):
    """
    This represents an estimated accel with reference coordinate frame and timestamp.
    """
    header: Header = default_field(Header)
    accel: AccelWithCovariance = default_field(AccelWithCovariance)


@dataclass
class Inertia(IdlStruct, typename='geometry_msgs/Inertia'):
    m: float64 = 0
    """
    Mass [kg]
    """

    com: Vector3 = default_field(Vector3)
    """
    Center of mass [m]
    """

    ixx: float64 = 0
    ixy: float64 = 0
    ixz: float64 = 0
    iyy: float64 = 0
    iyz: float64 = 0
    izz: float64 = 0
    """
    Inertia Tensor [kg-m^2]
        | ixx ixy ixz |
    I = | ixy iyy iyz |
        | ixz iyz izz |
    """


@dataclass
class InertiaStamped(IdlStruct, typename='geometry_msgs/InertiaStamped'):
    """
    An Inertia with a time stamp and reference frame.
    """
    header: Header = default_field(Header)
    inertia: Inertia = default_field(Inertia)


@dataclass
class Point(IdlStruct, typename='geometry_msgs/Point'):
    """
    This contains the position of a point in free space
    """
    x: float64 = 0
    y: float64 = 0
    z: float64 = 0


@dataclass
class Point32(IdlStruct, typename='geometry_msgs/Point32'):
    """
    This contains the position of a point in free space(with 32 bits of precision).
    It is recommended to use Point wherever possible instead of Point32.
    This recommendation is to promote interoperability.
    This message is designed to take up less space when sending
    lots of points at once, as in the case of a PointCloud.
    """
    x: float32 = 0
    y: float32 = 0
    z: float32 = 0


@dataclass
class PointStamped(IdlStruct, typename='geometry_msgs/PointStamped'):
    """
    This represents a Point with reference coordinate frame and timestamp
    """
    header: Header = default_field(Header)
    point: Point = default_field(Point)


@dataclass
class Polygon(IdlStruct, typename='geometry_msgs/Polygon'):
    """
    A specification of a polygon where the first and last points are assumed to be connected
    """
    points: sequence[Point32] = default_field([])


@dataclass
class PolygonStamped(IdlStruct, typename='geometry_msgs/PolygonStamped'):
    """
    This represents a Polygon with reference coordinate frame and timestamp
    """
    header: Header = default_field(Header)
    polygon: Polygon = default_field(Polygon)


@dataclass
class Pose(IdlStruct, typename='geometry_msgs/Pose'):
    """
    A representation of pose in free space, composed of position and orientation.
    """
    position: Point = default_field(Point)
    orientation: Quaternion = default_field(Quaternion)


@dataclass
class Pose2D(IdlStruct, typename='geometry_msgs/Pose2D'):
    """
    Deprecated as of Foxy and will potentially be removed in any following release.
    Please use the full 3D pose.

    In general our recommendation is to use a full 3D representation of everything and 
    for 2D specific applications make the appropriate projections into the plane for 
    their calculations but optimally will preserve the 3D information during processing.

    If we have parallel copies of 2D datatypes every UI and other pipeline will end up 
    needing to have dual interfaces to plot everything. And you will end up with 
    not being able to use 3D tools for 2D use cases even if they're completely 
    valid, as you'd have to reimplement it with different inputs and outputs. 
    It's not particularly hard to plot the 2D pose or compute the yaw error for 
    the Pose message and there are already tools and libraries that can do this for you.
    This expresses a position and orientation on a 2D manifold.
    """
    x: float64 = 0
    y: float64 = 0
    theta: float64 = 0


@dataclass
class PoseArray(IdlStruct, typename='geometry_msgs/PoseArray'):
    """
    An array of poses with a header for global reference.
    """
    header: Header = default_field(Header)

    poses: sequence[Pose] = default_field([])


@dataclass
class PoseStamped(IdlStruct, typename='geometry_msgs/PoseStamped'):
    """
    A Pose with reference coordinate frame and timestamp
    """
    header: Header = default_field(Header)
    pose: Pose = default_field(Pose)


@dataclass
class PoseWithCovariance(IdlStruct, typename='geometry_msgs/PoseWithCovariance'):
    """
    This represents a pose in free space with uncertainty.
    """
    pose: Pose = default_field(Pose)

    covariance: array[float64, 36] = default_field([0] * 36)
    """
    Row-major representation of the 6x6 covariance matrix
    The orientation parameters use a fixed-axis representation.
    In order, the parameters are:
    (x, y, z, rotation about X axis, rotation about Y axis, rotation about Z axis)
    """


@dataclass
class PoseWithCovarianceStamped(IdlStruct, typename='geometry_msgs/PoseWithCovarianceStamped'):
    """
    This expresses an estimated pose with a reference coordinate frame and timestamp
    """
    header: Header = default_field(Header)
    pose: PoseWithCovariance = default_field(PoseWithCovariance)


@dataclass
class Transform(IdlStruct, typename='geometry_msgs/Transform'):
    """
    This represents the transform between two coordinate frames in free space.
    """
    translation: Vector3 = default_field(Vector3)
    rotation: Quaternion = default_field(Quaternion)


@dataclass
class TransformStamped(IdlStruct, typename='geometry_msgs/TransformStamped'):
    """
    This expresses a transform from coordinate frame header.frame_id
    to the coordinate frame child_frame_id at the time of header.stamp

    This message is mostly used by the
    <a href="https://index.ros.org/p/tf2/">tf2</a> package.
    See its documentation for more information.

    The child_frame_id is necessary in addition to the frame_id
    in the Header to communicate the full reference for the transform
    in a self contained message.
    """

    header: Header = default_field(Header)
    """
    The frame id in the header is used as the reference frame of this transform.
    """

    child_frame_id: str = ''
    """
    The frame id of the child frame to which this transform points.
    """

    transform: Transform = default_field(Transform)
    """
    Translation and rotation in 3-dimensions of child_frame_id from header.frame_id.
    """


@dataclass
class Twist(IdlStruct, typename='geometry_msgs/Twist'):
    """
    This expresses velocity in free space broken into its linear and angular parts.
    """
    linear: Vector3 = default_field(Vector3)
    angular: Vector3 = default_field(Vector3)


@dataclass
class TwistStamped(IdlStruct, typename='geometry_msgs/TwistStamped'):
    """
    A twist with reference coordinate frame and timestamp
    """
    header: Header = default_field(Header)
    twist: Twist = default_field(Twist)


@dataclass
class TwistWithCovariance(IdlStruct, typename='geometry_msgs/TwistWithCovariance'):
    """
    This expresses velocity in free space with uncertainty.
    """
    twist: Twist = default_field(Twist)

    covariance: array[float64, 36] = default_field([0] * 36)
    """
    Row-major representation of the 6x6 covariance matrix
    The orientation parameters use a fixed-axis representation.
    In order, the parameters are:
    (x, y, z, rotation about X axis, rotation about Y axis, rotation about Z axis)
    """


@dataclass
class TwistWithCovarianceStamped(IdlStruct, typename='geometry_msgs/TwistWithCovarianceStamped'):
    """
    This represents an estimated twist with reference coordinate frame and timestamp.
    """
    header: Header = default_field(Header)
    twist: TwistWithCovariance = default_field(TwistWithCovariance)


@dataclass
class Wrench(IdlStruct, typename='geometry_msgs/Wrench'):
    """
    This represents force in free space, separated into its linear and angular parts.
    """
    force: Vector3 = default_field(Vector3)
    torque: Vector3 = default_field(Vector3)


@dataclass
class WrenchStamped(IdlStruct, typename='geometry_msgs/WrenchStamped'):
    """
    A wrench with reference coordinate frame and timestamp
    """
    header: Header = default_field(Header)
    wrench: Wrench = default_field(Wrench)


# Schema registry support
_TYPES = {
    "Accel": Accel,
    "AccelStamped": AccelStamped,
    "AccelWithCovariance": AccelWithCovariance,
    "AccelWithCovarianceStamped": AccelWithCovarianceStamped,
    "Inertia": Inertia,
    "InertiaStamped": InertiaStamped,
    "Point": Point,
    "Point32": Point32,
    "PointStamped": PointStamped,
    "Polygon": Polygon,
    "PolygonStamped": PolygonStamped,
    "Pose": Pose,
    "Pose2D": Pose2D,
    "PoseArray": PoseArray,
    "PoseStamped": PoseStamped,
    "PoseWithCovariance": PoseWithCovariance,
    "PoseWithCovarianceStamped": PoseWithCovarianceStamped,
    "Quaternion": Quaternion,
    "QuaternionStamped": QuaternionStamped,
    "Transform": Transform,
    "TransformStamped": TransformStamped,
    "Twist": Twist,
    "TwistStamped": TwistStamped,
    "TwistWithCovariance": TwistWithCovariance,
    "TwistWithCovarianceStamped": TwistWithCovarianceStamped,
    "Vector3": Vector3,
    "Vector3Stamped": Vector3Stamped,
    "Wrench": Wrench,
    "WrenchStamped": WrenchStamped,
}


def is_type_supported(type_name: str) -> bool:
    """Check if a type name is supported by this module."""
    return type_name in _TYPES


def list_types() -> list[str]:
    """List all type schema names in this module."""
    return [f"geometry_msgs/msg/{name}" for name in sorted(_TYPES.keys())]


def get_type(type_name: str) -> type:
    """Get the type class by name. Returns None if not found."""
    return _TYPES.get(type_name)



# SPDX-License-Identifier: Apache-2.0
# Copyright © 2025 Au-Zone Technologies. All Rights Reserved.

from dataclasses import dataclass
from enum import Enum

from pycdr2 import IdlStruct
from pycdr2.types import array, float64, sequence, uint8, uint32

from . import default_field
from .builtin_interfaces import Duration, Time
from .geometry_msgs import Point, Pose, Quaternion, Vector3


class FieldDatatype(Enum):
    UNKNOWN = 0
    UINT8 = 1
    INT8 = 2
    UINT16 = 3
    INT16 = 4
    UINT32 = 5
    INT32 = 6
    FLOAT32 = 7
    FLOAT64 = 8


@dataclass
class PackedElementField(IdlStruct, typename='foxglove_msgs/PackedElementField'):
    """
    foxglove_msgs/msg/PackedElementField
    A field present within each element in a byte array of packed elements.

    Generated by https://github.com/foxglove/schemas
    """
    name: str = ''
    """
    Name of the field
    """

    offset: uint32 = 0
    """
    Byte offset from start of data buffer
    """

    type: uint8 = 0
    """
    Type of data in the field. Integers are stored using little-endian byte order.
    """


@dataclass
class Vector2(IdlStruct, typename='foxglove_msgs/Vector2'):
    """
    foxglove_msgs/msg/Vector2
    A vector in 2D space that represents a direction only

    Generated by https://github.com/foxglove/schemas
    """
    x: float64 = 0
    """
    x coordinate length
    """

    y: float64 = 0
    """
    y coordinate length
    """


@dataclass
class Color(IdlStruct, typename='foxglove_msgs/Color'):
    """
    foxglove_msgs/msg/Color
    A color in RGBA format

    Generated by https://github.com/foxglove/schemas
    """
    r: float64 = 0
    """
    Red value between 0 and 1
    """
    g: float64 = 0
    """
    Green value between 0 and 1
    """
    b: float64 = 0
    """
    Blue value between 0 and 1
    """
    a: float64 = 0
    """
    Alpha value between 0 and 1
    """


@dataclass
class Point2(IdlStruct, typename='foxglove_msgs/Point2'):
    """
    foxglove_msgs/msg/Point2
    A point representing a position in 2D space

    Generated by https://github.com/foxglove/schemas
    """
    x: float64 = 0
    """
    x coordinate position
    """
    y: float64 = 0
    """
    y coordinate position
    """


@dataclass
class KeyValuePair(IdlStruct, typename='foxglove_msgs/KeyValuePair'):
    """
    foxglove_msgs/msg/KeyValuePair
    A key with its associated value

     Generated by https://github.com/foxglove/schemas"""
    key: str = ''
    """
    Key
    """
    value: str = ''
    """
    Value
    """


@dataclass
class ArrowMarker(IdlStruct, typename='foxglove_msgs/ArrowMarker'):
    """
    foxglove_msgs/msg/ArrowMarker
    A marker representing an arrow

    Generated by https://github.com/foxglove/schemas
    """
    pose: Pose = default_field(Pose)
    """
    Position of the arrow's tail and orientation of the arrow. Identity orientation means the arrow points in the +x direction.
    """

    length: float64 = 0
    """
    Length of the arrow
    """
    shaft_diameter: float64 = 0
    """
    Diameter of the arrow shaft
    """
    head_diameter: float64 = 0
    """
    Diameter of the arrow head
    """
    head_length: float64 = 0
    """
    Length of the arrow head
    """
    color: Color = default_field(Color)
    """
    Color of the arrow
    """


@dataclass
class ArrowPrimitive(IdlStruct, typename='foxglove_msgs/ArrowPrimitive'):
    """
    foxglove_msgs/msg/ArrowPrimitive
    A primitive representing an arrow

    Generated by https://github.com/foxglove/schemas
    """
    pose: Pose = default_field(Pose)
    """
    Position of the arrow's tail and orientation of the arrow. Identity orientation means the arrow points in the +x direction.
    """
    shaft_length: float64 = 0
    """
    Length of the arrow shaft
    """
    shaft_diameter: float64 = 0
    """
    Diameter of the arrow shaft
    """
    head_length: float64 = 0
    """
    Length of the arrow head
    """
    head_diameter: float64 = 0
    """
    Diameter of the arrow head
    """
    color: Color = default_field(Color)
    """
    Color of the arrow
    """


@dataclass
class CameraCalibration(IdlStruct, typename='foxglove_msgs/CameraCalibration'):
    """
    foxglove_msgs/msg/CameraCalibration
    Camera calibration parameters

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of calibration data
    """
    frame_id: str = ''
    """
    Frame of reference for the camera. The origin of the frame is the optical center of the camera. 
    +x points to the right in the image, +y points down, and +z points into the plane of the image.
    """
    width: uint32 = 0
    """
    Image width
    """
    height: uint32 = 0
    """
    Image height
    """

    distortion_model: str = ''
    """
    Name of distortion model
     
    Supported parameters: `plumb_bob` (k1, k2, p1, p2, k3) and `rational_polynomial` (k1, k2, p1, p2, k3, k4, k5, k6). 
    Distortion models are based on [OpenCV's](https://docs.opencv.org/2.4/modules/calib3d/doc/camera_calibration_and_3d_reconstruction.html) [pinhole camera model](https://en.wikipedia.org/wiki/Distortion_%28optics%29#Software_correction). 
    This is the same [implementation used by ROS](http://docs.ros.org/en/diamondback/api/image_geometry/html/c++/pinhole__camera__model_8cpp_source.html)
    """

    d: sequence[float64] = default_field([])
    """
    Distortion parameters
    """

    k: array[float64, 9] = default_field([0] * 9)
    """
    Intrinsic camera matrix (3x3 row-major matrix)
    
    A 3x3 row-major matrix for the raw (distorted) image.
    
    Projects 3D points in the camera coordinate frame to 2D pixel coordinates using the focal lengths (fx, fy) and principal point (cx, cy).
     
    ```
        [fx  0 cx]
    K = [ 0 fy cy]
        [ 0  0  1]
    ```
    """

    r: array[float64, 9] = default_field([0] * 9)
    """
    Rectification matrix (stereo cameras only, 3x3 row-major matrix)
     
    A rotation matrix aligning the camera coordinate system to the ideal stereo image plane so that epipolar lines in both stereo images are parallel.
    """

    p: array[float64, 12] = default_field([0] * 12)
    """
    Projection/camera matrix (3x4 row-major matrix)
    
    ```
        [fx'  0  cx' Tx]
    P = [ 0  fy' cy' Ty]
        [ 0   0   1   0]
    ```
    
    By convention, this matrix specifies the intrinsic (camera) matrix of the processed (rectified) image. That is, the left 3x3 portion is the normal camera intrinsic matrix for the rectified image.
     
    It projects 3D points in the camera coordinate frame to 2D pixel coordinates using the focal lengths (fx', fy') and principal point (cx', cy') - these may differ from the values in K.
     
    For monocular cameras, Tx = Ty = 0. Normally, monocular cameras will also have R = the identity and P[1:3,1:3] = K.
     
    For a stereo pair, the fourth column [Tx Ty 0]' is related to the position of the optical center of the second camera in the first camera's frame. We assume Tz = 0 so both cameras are in the same stereo image plane. The first camera always has Tx = Ty = 0. For the right (second) camera of a horizontal stereo pair, Ty = 0 and Tx = -fx' * B, where B is the baseline between the cameras.
    
    Given a 3D point [X Y Z]', the projection (x, y) of the point onto the rectified image is given by:
    
    ```
    [u v w]' = P * [X Y Z 1]'
           x = u / w
           y = v / w
    ```
    
    This holds for both images of a stereo pair.
    """


@dataclass
class CircleAnnotation(IdlStruct, typename='foxglove_msgs/CircleAnnotation'):
    """
    foxglove_msgs/msg/CircleAnnotation
    A circle annotation on a 2D image

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of circle
    """
    position: Point2 = default_field(Point2)
    """
    Center of the circle in 2D image coordinates (pixels)
    """

    diameter: float64 = 0
    """
    Circle diameter in pixels
    """
    thickness: float64 = 0
    """
    Line thickness in pixels
    """
    fill_color: Color = default_field(Color)
    """
    Fill color
    """
    outline_color: Color = default_field(Color)
    """
    Outline color
    """


@dataclass
class CompressedImage(IdlStruct, typename='foxglove_msgs/CompressedImage'):
    """
    foxglove_msgs/msg/CompressedImage
    A compressed image

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of image
    """
    frame_id: str = ''
    """
    Frame of reference for the image. The origin of the frame is the optical center of the camera. +x points to the right in the image, +y points down, and +z points into the plane of the image.
    """

    data: sequence[uint8] = default_field([])
    """
    Compressed image data
    """

    format: str = ''
    """
    Image format
    
    Supported values: image media types supported by Chrome, such as `webp`, `jpeg`, `png`
    """


@dataclass
class CompressedVideo(IdlStruct, typename='foxglove_msgs/CompressedVideo'):
    """
    foxglove_msgs/msg/CompressedVideo
    A single frame of a compressed video bitstream

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of video frame
    """

    frame_id: str = ''
    """
    Frame of reference for the video.
    
    The origin of the frame is the optical center of the camera. +x points to the right in the video, +y points down, and +z points into the plane of the video.
    """

    data: sequence[uint8] = default_field([])
    """
    Compressed video frame data.
    
    For packet-based video codecs this data must begin and end on packet boundaries (no partial packets), and must contain enough video packets to decode exactly one image (either a keyframe or delta frame). Note: Foxglove does not support video streams that include B frames because they require lookahead.
    """

    format: str = ''
    """
    Video format.
    
    Supported values: `h264` (Annex B formatted data only)
    """


@dataclass
class ConeAttributes(IdlStruct, typename='foxglove_msgs/ConeAttribute'):
    """
    foxglove_msgs/msg/ConeAttributes
    Data specifying the visual appearance of a possibly truncated, possibly elliptic cone or cylinder

    Generated by https://github.com/foxglove/schemas
    """
    pose: Pose = default_field(Pose)
    """
    Position of the center of the cone and orientation of the cone. The flat face(s) are perpendicular to the z-axis.
    """
    size: Vector3 = default_field(Vector3)
    """
    Size of the cone's bounding box
    """
    bottom_scale: float64 = 0
    """
    0-1, size of the cone's bottom face (min z) relative to the bottom of the bounding box
    """
    top_scale: float64 = 0
    """
    0-1, size of the cone's top face (max z) relative to the top of the bounding box
    """
    color: Color = default_field(Color)
    """
    Color of the cone
    """


@dataclass
class ConeListMarker(IdlStruct, typename='foxglove_msgs/ConeListMarker'):
    """
    foxglove_msgs/msg/ConeListMarker
    A marker representing a list of possibly truncated, possibly elliptic cones or cylinders

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of the marker
    """
    frame_id: str = ''
    """
    Frame of reference
    """
    id: str = ''
    """
    Identifier for the marker. A marker will replace any prior marker on the same topic with the same `id`.
    """
    lifetime: Duration = default_field(Duration)
    """
    Length of time (relative to `timestamp`) after which the marker should be automatically removed. Zero value indicates the marker should remain visible until it is replaced or deleted.
    """
    frame_locked: bool = False
    """
    Whether the marker should keep its location in the fixed frame (false) or follow the frame specified in `frame_id` as it moves relative to the fixed frame (true)
    """
    metadata: sequence[KeyValuePair] = default_field([])
    """
    Additional user-provided metadata associated with the marker. Keys must be unique.
    """
    attributes: sequence[ConeAttributes] = default_field([])
    """
    Attributes of each cone
    """


@dataclass
class ConeMarker(IdlStruct, typename='foxglove_msgs/ConeMarker'):
    """
    foxglove_msgs/msg/ConeMarker
    A marker representing a possibly truncated, possibly elliptic cone or cylinder

    Generated by https://github.com/foxglove/schemas
    """
    pose: Pose = default_field(Pose)
    """
    Position of the center of the cone and orientation of the cone. The flat face(s) are perpendicular to the z-axis.
    """
    size: Vector3 = default_field(Vector3)
    """
    Size of the cone's bounding box
    """
    bottom_scale: float64 = 0
    """
    0-1, size of the cone's bottom face (min z) relative to the bottom of the bounding box
    """
    top_scale: float64 = 0
    """
    0-1, size of the cone's top face (max z) relative to the top of the bounding box
    """
    color: Color = default_field(Color)
    """
    Color of the cone
    """


@dataclass
class ConePrimitive(IdlStruct, typename='foxglove_msgs/ConePrimitive'):
    """
    foxglove_msgs/msg/ConePrimitive
    A primitive representing a possibly truncated, possibly elliptic cone or cylinder

    Generated by https://github.com/foxglove/schemas
    """
    pose: Pose = default_field(Pose)
    """
    Position of the center of the cone and orientation of the cone. The flat face(s) are perpendicular to the z-axis.
    """
    size: Vector3 = default_field(Vector3)
    """
    Size of the cone's bounding box
    """
    bottom_scale: float64 = 0
    """
    0-1, size of the cone's bottom face (min z) relative to the bottom of the bounding box
    """
    top_scale: float64 = 0
    """
    0-1, size of the cone's top face (max z) relative to the top of the bounding box
    """
    color: Color = default_field(Color)
    """
    Color of the cone
    """


@dataclass
class CubeAttributes(IdlStruct, typename='foxglove_msgs/CubeAttributes'):
    """
    foxglove_msgs/msg/CubeAttributes
    Data specifying the visual appearance of a cube or rectangular prism

    Generated by https://github.com/foxglove/schemas
    """
    pose: Pose = default_field(Pose)
    """
    Position of the center of the cube and orientation of the cube
    """
    size: Vector3 = default_field(Vector3)
    """
    Size of the cube along each axis
    """
    color: Color = default_field(Color)
    """
    Color of the arrow
    """


@dataclass
class CubeListMarker(IdlStruct, typename='foxglove_msgs/CubeListMarker'):
    """
    foxglove_msgs/msg/CubeListMarker
    A marker representing a list of cubes or rectangular prisms

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of the marker
    """
    frame_id: str = ''
    """
    Frame of reference
    """
    id: str = ''
    """
    Identifier for the marker. A marker will replace any prior marker on the same topic with the same `id`.
    """
    lifetime: Duration = default_field(Duration)
    """
    Length of time (relative to `timestamp`) after which the marker should be automatically removed. Zero value indicates the marker should remain visible until it is replaced or deleted.
    """
    frame_locked: bool = False
    """
    Whether the marker should keep its location in the fixed frame (false) or follow the frame specified in `frame_id` as it moves relative to the fixed frame (true)
    """
    metadata: sequence[KeyValuePair] = default_field([])
    """
    Additional user-provided metadata associated with the marker. Keys must be unique.
    """
    attributes: sequence[CubeAttributes] = default_field([])
    """
    Attributes of each cube
    """


@dataclass
class CubeMarker(IdlStruct, typename='foxglove_msgs/CubeMarker'):
    """
    foxglove_msgs/msg/CubeMarker
    A marker representing a cube or rectangular prism

    Generated by https://github.com/foxglove/schemas
    """
    pose: Pose = default_field(Pose)
    """
    Position of the center of the cube and orientation of the cube
    """
    size: Vector3 = default_field(Vector3)
    """
    Size of the cube along each axis
    """
    color: Color = default_field(Color)
    """
    Color of the arrow
    """


@dataclass
class CubePrimitive(IdlStruct, typename='foxglove_msgs/CubePrimitive'):
    """
    foxglove_msgs/msg/CubePrimitive
    A primitive representing a cube or rectangular prism

    Generated by https://github.com/foxglove/schemas
    """
    pose: Pose = default_field(Pose)
    """
    Position of the center of the cube and orientation of the cube
    """
    size: Vector3 = default_field(Vector3)
    """
    Size of the cube along each axis
    """
    color: Color = default_field(Color)
    """
    Color of the cube
    """


@dataclass
class CylinderMarker(IdlStruct, typename='foxglove_msgs/CylinderMarker'):
    """
    foxglove_msgs/msg/CylinderMarker
    A marker representing a cylinder or elliptic cylinder

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of the marker
    """

    frame_id: str = ''
    """
    Frame of reference
    """

    id: str = ''
    """
    Identifier for the marker. A marker will replace any prior marker on the same topic with the same `id`.
    """

    lifetime: Duration = default_field(Duration)
    """
    Length of time (relative to `timestamp`) after which the marker should be automatically removed. Zero value indicates the marker should remain visible until it is replaced or deleted.
    """

    frame_locked: bool = False
    """
    Whether the marker should keep its location in the fixed frame (false) or follow the frame specified in `frame_id` as it moves relative to the fixed frame (true)
    """

    metadata: sequence[KeyValuePair] = default_field([])
    """
    Additional user-provided metadata associated with the marker. Keys must be unique.
    """

    pose: Pose = default_field(Pose)
    """
    Position of the center of the cylinder and orientation of the cylinder. The cylinder's flat faces are perpendicular to the z-axis.
    """

    bottom_radius: float64 = 0
    """
    Radius of the cylinder at min z
    """

    top_radius: float64 = 0
    """
    Radius of the cylinder at max z
    """

    height: float64 = 0
    """
    Height of the cylinder along the z axis
    """

    color: Color = default_field(Color)
    """
    Color of the sphere
    """


@dataclass
class CylinderPrimitive(IdlStruct, typename='foxglove_msgs/CylinderPrimitive'):
    """
    foxglove_msgs/msg/CylinderPrimitive
    A primitive representing a cylinder, elliptic cylinder, or truncated cone

    Generated by https://github.com/foxglove/schemas
    """
    pose: Pose = default_field(Pose)
    """
    Position of the center of the cylinder and orientation of the cylinder. The flat face(s) are perpendicular to the z-axis.
    """

    size: Vector3 = default_field(Vector3)
    """
    Size of the cylinder's bounding box
    """

    bottom_scale: float64 = 0
    """
    0-1, ratio of the diameter of the cylinder's bottom face (min z) to the bottom of the bounding box
    """

    top_scale: float64 = 0
    """
    0-1, ratio of the diameter of the cylinder's top face (max z) to the top of the bounding box
    """

    color: Color = default_field(Color)
    """
    Color of the cylinder
    """


@dataclass
class FrameTransform(IdlStruct, typename='foxglove_msgs/FrameTransform'):
    """
    foxglove_msgs/msg/FrameTransform
    A transform between two reference frames in 3D space

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of transform
    """

    parent_frame_id: str = ''
    """
    Name of the parent frame
    """

    child_frame_id: str = ''
    """
    Name of the child frame
    """

    translation: Vector3 = default_field(Vector3)
    """
    Translation component of the transform
    """

    rotation: Quaternion = default_field(Quaternion)
    """
    Rotation component of the transform
    """


@dataclass
class FrameTransformList(IdlStruct, typename='foxglove_msgs/FrameTransformList'):
    """
    foxglove_msgs/msg/FrameTransformList
    A list of transforms between reference frames in 3D space

    Generated by https://github.com/foxglove/schemas
    """
    transforms: sequence[FrameTransform] = default_field([])
    """
    List of transforms
    """


@dataclass
class FrameTransforms(IdlStruct, typename='foxglove_msgs/FrameTransforms'):
    """
    foxglove_msgs/msg/FrameTransforms
    An array of FrameTransform messages

    Generated by https://github.com/foxglove/schemas
    """
    transforms: sequence[FrameTransform] = default_field([])
    """
    Array of transforms
    """


@dataclass
class GeoJSON(IdlStruct, typename='foxglove_msgs/GeoJSON'):
    """
    foxglove_msgs/msg/GeoJSON
    GeoJSON data for annotating maps

    Generated by https://github.com/foxglove/schemas
    """
    geojson: str = ''
    """
    GeoJSON data encoded as a UTF-8 string
    """


@dataclass
class Grid(IdlStruct, typename='foxglove_msgs/Grid'):
    """
    foxglove_msgs/msg/Grid
    A 2D grid of data

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of grid
    """

    frame_id: str = ''
    """
    Frame of reference
    """

    pose: Pose = default_field(Pose)
    """
    Origin of grid's corner relative to frame of reference; grid is positioned in the x-y plane relative to this origin
    """

    column_count: uint32 = 0
    """
    Number of grid columns
    """

    cell_size: Vector2 = default_field(Vector2)
    """
    Size of single grid cell along x and y axes, relative to `pose`
    """

    row_stride: uint32 = 0
    """
    Number of bytes between rows in `data`
    """

    cell_stride: uint32 = 0
    """
    Number of bytes between cells within a row in `data`
    """

    fields: sequence[PackedElementField] = default_field([])
    """
    Fields in `data`. `red`, `green`, `blue`, and `alpha` are optional for customizing the grid's color.
    """

    data: uint8 = 0
    """
    Grid cell data, interpreted using `fields`, in row-major (y-major) order
    """


class PointType(Enum):
    UNKNOWN = 0
    POINTS = 1
    """
    Individual points: 0, 1, 2, ...
    """
    LINE_LOOP = 2
    """
    Closed polygon: 0-1, 1-2, ..., (n-1)-n, n-0
    """
    LINE_STRIP = 3
    """
    Connected line segments: 0-1, 1-2, ..., (n-1)-n
    """
    LINE_LIST = 4
    """
    Individual line segments: 0-1, 2-3, 4-5, ...
    """


@dataclass
class PointsAnnotation(IdlStruct, typename='foxglove_msgs/PointsAnnotation'):
    """
    foxglove_msgs/msg/PointsAnnotation
    An array of points on a 2D image

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of annotation
    """

    type: uint8 = 0
    """
    Type of points annotation to draw
    """

    points: sequence[Point2] = default_field([])
    """
    Points in 2D image coordinates (pixels)
    """

    outline_color: Color = default_field(Color)
    """
    Outline color
    """

    outline_colors: sequence[Color] = default_field([])
    """
    Per-point colors, if `type` is `POINTS`, or per-segment stroke colors, if `type` is `LINE_LIST`, `LINE_STRIP` or `LINE_LOOP`.
    """

    fill_color: Color = default_field(Color)
    """
    Fill color
    """

    thickness: float64 = 0
    """
    Stroke thickness in pixels
    """


@dataclass
class TextAnnotation(IdlStruct, typename='foxglove_msgs/TextAnnotation'):
    """
    foxglove_msgs/msg/TextAnnotation
    A text label on a 2D image

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of annotation
    """

    position: Point2 = default_field(Point2)
    """
    Bottom-left origin of the text label in 2D image coordinates (pixels)
    """

    text: str = ''
    """
    Text to display
    """

    font_size: float64 = 0
    """
    Font size in pixels
    """

    text_color: Color = default_field(Color)
    """
    Text color
    """

    background_color: Color = default_field(Color)
    """
    Background fill color
    """


@dataclass
class ImageAnnotations(IdlStruct, typename='foxglove_msgs/ImageAnnotations'):
    """
    foxglove_msgs/msg/ImageAnnotations
    Array of annotations for a 2D image

    Generated by https://github.com/foxglove/schemas
    """
    circles: sequence[CircleAnnotation] = default_field([])
    """
    Circle annotations
    """

    points: sequence[PointsAnnotation] = default_field([])
    """
    Points annotations
    """

    texts: sequence[TextAnnotation] = default_field([])
    """
    Text annotations
    """


@dataclass
class LaserScan(IdlStruct, typename='foxglove_msgs/LaserScan'):
    """
    foxglove_msgs/msg/LaserScan
    A single scan from a planar laser range-finder

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of scan
    """

    frame_id: str = ''
    """
    Frame of reference
    """

    pose: Pose = default_field(Pose)
    """
    Origin of scan relative to frame of reference; points are positioned in the x-y plane relative to this origin; angles are interpreted as counterclockwise rotations around the z axis with 0 rad being in the +x direction
    """

    start_angle: float64 = 0
    """
    Origin of scan relative to frame of reference; points are positioned in the x-y plane relative to this origin; angles are interpreted as counterclockwise rotations around the z axis with 0 rad being in the +x direction
    """

    end_angle: float64 = 0
    """
    Bearing of last point, in radians
    """

    ranges: sequence[float64] = default_field([])
    """
    Distance of detections from origin; assumed to be at equally-spaced angles between `start_angle` and `end_angle`
    """

    intensities: sequence[float64] = default_field([])
    """
    Intensity of detections
    """


class LinePrimitive(Enum):
    # 0-1, 1-2, ..., (n-1)-n
    LINE_STRIP = 0
    # 0-1, 1-2, ..., (n-1)-n, n-0
    LINE_LOOP = 1
    # 0-1, 2-3, 4-5, ...
    LINE_LIST = 2


@dataclass
class LineMarker(IdlStruct, typename='foxglove_msgs/LineMarker'):
    """
    foxglove_msgs/msg/LineMarker
    A marker representing a series of points connected by lines

    Generated by https://github.com/foxglove/schemas
    """
    type: uint8 = 0
    """
    Drawing primitive to use for lines
    """

    pose: Pose = default_field(Pose)
    """
    Origin of lines relative to reference frame
    """

    thickness: float64 = 0
    """
    Line thickness
    """

    scale_invariant: bool = False
    """
    Indicates whether `thickness` is a fixed size in screen pixels (true), or specified in world coordinates and scales with distance from the camera (false)
    """

    points: sequence[Point] = default_field([])
    """
    Points along the line
    """

    color: Color = default_field(Color)
    """
    Solid color to use for the whole line. One of `color` or `colors` must be provided.
    """

    colors: sequence[Color] = default_field([])
    """
    Per-point colors (if specified, must have the same length as `points`). One of `color` or `colors` must be provided.
    """

    indices: sequence[uint32] = default_field([])
    """
    Indices into the `points` and `colors` attribute arrays, which can be used to avoid duplicating attribute data.
     
    If omitted or empty, indexing will not be used. This default behavior is equivalent to specifying [0, 1, ..., N-1] for the indices (where N is the number of `points` provided).
    """


class LinePrimitiveType(Enum):
    LINE_STRIP = 0
    """
    Connected line segments: 0-1, 1-2, ..., (n-1)-n
    """
    LINE_LOOP = 1
    """
    Closed polygon: 0-1, 1-2, ..., (n-1)-n, n-0
    """
    LINE_LIST = 2
    """
    Individual line segments: 0-1, 2-3, 4-5, ...
    """


@dataclass
class LinePrimitive(IdlStruct, typename='foxglove_msgs/LinePrimitive'):
    """
    foxglove_msgs/msg/LinePrimitive
    A primitive representing a series of points connected by lines

    Generated by https://github.com/foxglove/schemas
    """
    type: uint8 = 0
    """
    Drawing primitive to use for lines
    """

    pose: Pose = default_field(Pose)
    """
    Origin of lines relative to reference frame
    """

    thickness: float64 = 0
    """
    Line thickness
    """

    scale_invariant: bool = False
    """
    Indicates whether `thickness` is a fixed size in screen pixels (true), or specified in world coordinates and scales with distance from the camera (false)
    """

    points: sequence[Point] = default_field([])
    """
    Points along the line
    """

    color: Color = default_field(Color)
    """
    Solid color to use for the whole line. One of `color` or `colors` must be provided.
    """

    colors: sequence[Color] = default_field([])
    """
    Per-point colors (if specified, must have the same length as `points`). One of `color` or `colors` must be provided.
    """

    indices: sequence[uint32] = default_field([])
    """
    Indices into the `points` and `colors` attribute arrays, which can be used to avoid duplicating attribute data.
    
    If omitted or empty, indexing will not be used. This default behavior is equivalent to specifying [0, 1, ..., N-1] for the indices (where N is the number of `points` provided).
    """


class PositionCovarianceType(Enum):
    UNKNOWN = 0
    APPROXIMATED = 1
    DIAGONAL_KNOWN = 2
    KNOWN = 3


@dataclass
class LocationFix(IdlStruct, typename='foxglove_msgs/LocationFix'):
    """
    foxglove_msgs/msg/LocationFix
    A navigation satellite fix for any Global Navigation Satellite System

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of the message
    """

    frame_id: str = ''
    """
    Frame for the sensor. Latitude and longitude readings are at the origin of the frame.
    """

    latitude: float64 = 0
    """
    Latitude in degrees
    """

    longitude: float64 = 0
    """
    Longitude in degrees
    """

    altitude: float64 = 0
    """
    Altitude in meters
    """

    position_covariance: array[float64, 9] = default_field([0] * 9)
    """
    Position covariance (m^2) defined relative to a tangential plane through the reported position. The components are East, North, and Up (ENU), in row-major order.
    """

    position_covariance_type: uint8 = 0
    """
    If `position_covariance` is available, `position_covariance_type` must be set to indicate the type of covariance.
    """


class LogLevel(Enum):
    UNKNOWN = 0
    DEBUG = 1
    INFO = 2
    WARNING = 3
    ERROR = 4
    FATAL = 5


@dataclass
class Log(IdlStruct, typename='foxglove_msgs/Log'):
    """
    foxglove_msgs/msg/Log
    A log message

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of log message
    """

    level: uint8 = 0
    """
    Log level
    """

    message: str = ''
    """
    Log message
    """

    name: str = ''
    """
    Process or node name
    """

    file: str = ''
    """
    Filename
    """

    line: uint32 = 0
    """
    Line number in the file
    """


class DeleteType(Enum):
    MATCHING_ID = 0
    ALL = 1


@dataclass
class MarkerDeletion(IdlStruct, typename='foxglove_msgs/MarkerDeletion'):
    """
    foxglove_msgs/msg/MarkerDeletion
    Command to remove previously published markers

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of the marker. Only matching markers earlier than this timestamp will be deleted.
    """

    type: uint8 = 0
    """
    Type of deletion action to perform
    """

    id: str = ''
    """
    Numeric identifier which must match if `kind` is `MATCHING_ID`.
    """


@dataclass
class ModelMarker(IdlStruct, typename='foxglove_msgs/ModelMarker'):
    """
    foxglove_msgs/msg/ModelMarker
    A marker representing a 3D model

    Generated by https://github.com/foxglove/schemas
    """
    pose: Pose = default_field(Pose)
    """
    Origin of model relative to reference frame
    """

    scale: Vector3 = default_field(Vector3)
    """
    Scale factor to apply to the model along each axis
    """

    color: Color = default_field(Color)
    """
    Solid color to use for the whole model. If `use_embedded_materials` is true, this color is blended on top of the embedded material color.
    """

    use_embedded_materials: bool = False
    """
    Whether to use materials embedded in the model, or only the `color`
    """

    url: str = ''
    """
    URL pointing to model file. Either `url` or `mime_type` and `data` should be provided.
    """

    mime_type: str = ''
    """
    MIME type of embedded model (e.g. `model/gltf-binary`). Either `url` or `mime_type` and `data` should be provided.
    """

    data: sequence[uint8] = default_field([])
    """
    Embedded model. Either `url` or `mime_type` and `data` should be provided.
    """


@dataclass
class SphereAttributes(IdlStruct, typename='foxglove_msgs/SphereAttributes'):
    """
    foxglove_msgs/msg/SphereAttributes
    Data specifying the visual appearance of a sphere or ellipsoid

    Generated by https://github.com/foxglove/schemas
    """
    pose: Pose = default_field(Pose)
    """
    Position of the center of the sphere and orientation of the sphere
    """

    size: Vector3 = default_field(Vector3)
    """
    Size (diameter) of the sphere along each axis
    """

    color: Color = default_field(Color)
    """
    Color of the sphere
    """


@dataclass
class SphereListMarker(IdlStruct, typename='foxglove_msgs/SphereListMarker'):
    """
    foxglove_msgs/msg/SphereListMarker
    A marker representing a list of spheres or ellipsoids

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of the marker
    """

    frame_id: str = ''
    """
    Frame of reference
    """

    id: str = ''
    """
    Identifier for the marker. A marker will replace any prior marker on the same topic with the same `id`.
    """

    lifetime: Duration = default_field(Duration)
    """
    Length of time (relative to `timestamp`) after which the marker should be automatically removed. Zero value indicates the marker should remain visible until it is replaced or deleted.
    """

    frame_locked: bool = False
    """
    Whether the marker should keep its location in the fixed frame (false) or follow the frame specified in `frame_id` as it moves relative to the fixed frame (true)
    """

    metadata: sequence[KeyValuePair] = default_field([])
    """
    Additional user-provided metadata associated with the marker. Keys must be unique.
    """

    attributes: sequence[SphereAttributes] = default_field([])
    """
    Attributes of each sphere
    """


@dataclass
class TextMarker(IdlStruct, typename='foxglove_msgs/TextMarker'):
    """
    foxglove_msgs/msg/TextMarker
    A marker representing a text label

    Generated by https://github.com/foxglove/schemas
    """
    pose: Pose = default_field(Pose)
    """
    Position of the center of the text box and orientation of the text. Identity orientation means the text is oriented in the xy-plane and flows from -x to +x.
    """

    billboard: bool = False
    """
    Whether the text should respect `pose.orientation` (false) or always face the camera (true)
    """

    font_size: float64 = 0
    """
    Font size (height of one line of text)
    """

    scale_invariant: bool = False
    """
    Indicates whether `font_size` is a fixed size in screen pixels (true), or specified in world coordinates and scales with distance from the camera (false)
    """

    color: Color = default_field(Color)
    """
    Color of the text
    """

    text: str = ''
    """
    Text
    """


@dataclass
class TriangleListMarker(IdlStruct, typename='foxglove_msgs/TriangleListMarker'):
    """
    foxglove_msgs/msg/TriangleListMarker
    A marker representing a set of triangles or a surface tiled by triangles

    Generated by https://github.com/foxglove/schemas
    """
    pose: Pose = default_field(Pose)
    """
    Origin of triangles relative to reference frame
    """

    points: sequence[Point] = default_field([])
    """
    Vertices to use for triangles, interpreted as a list of triples (0-1-2, 3-4-5, ...)
    """

    color: Color = default_field(Color)
    """
    Solid color to use for the whole shape. One of `color` or `colors` must be provided.
    """

    colors: sequence[Color] = default_field([])
    """
    Per-vertex colors (if specified, must have the same length as `points`). One of `color` or `colors` must be provided.
    """

    indices: sequence[uint32] = default_field([])
    """
    Indices into the `points` and `colors` attribute arrays, which can be used to avoid duplicating attribute data.
    
    If omitted or empty, indexing will not be used. This default behavior is equivalent to specifying [0, 1, ..., N-1] for the indices (where N is the number of `points` provided).
    """


@dataclass
class Markers(IdlStruct, typename='foxglove_msgs/Markers'):
    """
    foxglove_msgs/msg/Markers
    A list of any number or type of markers

    Generated by https://github.com/foxglove/schemas
    """
    deletions: sequence[MarkerDeletion] = default_field([])
    """
    Marker deletion actions
    """

    arrows: sequence[ArrowMarker] = default_field([])
    """
    Arrow markers
    """

    cubes: sequence[CubeListMarker] = default_field([])
    """
    Cube list markers
    """

    spheres: sequence[SphereListMarker] = default_field([])
    """
    Sphere list markers
    """

    cones: sequence[ConeListMarker] = default_field([])
    """
    Cone list markers
    """

    lines: sequence[LineMarker] = default_field([])
    """
    Line markers
    """

    triangles: sequence[TriangleListMarker] = default_field([])
    """
    Triangle list markers
    """

    texts: sequence[TextMarker] = default_field([])
    """
    Text markers
    """

    models: sequence[ModelMarker] = default_field([])
    """
    Model markers
    """


@dataclass
class ModelPrimitive(IdlStruct, typename='foxglove_msgs/ModelPrimitive'):
    """
    foxglove_msgs/msg/ModelPrimitive
    A primitive representing a 3D model file loaded from an external URL or embedded data

    Generated by https://github.com/foxglove/schemas
    """
    pose: Pose = default_field(Pose)
    """
    Origin of model relative to reference frame
    """

    scale: Vector3 = default_field(Vector3)
    """
    Scale factor to apply to the model along each axis
    """

    color: Color = default_field(Color)
    """
    Solid color to use for the whole model if `override_color` is true.
    """

    override_color: bool = False
    """
    Whether to use the color specified in `color` instead of any materials embedded in the original model.
    """

    url: str = ''
    """
    URL pointing to model file. One of `url` or `data` should be provided.
    """

    media_type: str = ''
    """
    [Media type](https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/MIME_types) of embedded model (e.g. `model/gltf-binary`). Required if `data` is provided instead of `url`. Overrides the inferred media type if `url` is provided.
    """

    data: sequence[uint8] = default_field([])
    """
    Embedded model. One of `url` or `data` should be provided. If `data` is provided, `media_type` must be set to indicate the type of the data.
    """


@dataclass
class PointCloud(IdlStruct, typename='foxglove_msgs/PointCloud'):
    """
    foxglove_msgs/msg/PointCloud
    A collection of N-dimensional points, which may contain additional fields with information like normals, intensity, etc.

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of point cloud
    """

    frame_id: str = ''
    """
    Frame of reference
    """

    pose: Pose = default_field(Pose)
    """
    The origin of the point cloud relative to the frame of reference
    """

    point_stride: uint32 = 0
    """
    Number of bytes between points in the `data`
    """

    fields: sequence[PackedElementField] = default_field([])
    """
    Fields in `data`. At least 2 coordinate fields from `x`, `y`, and `z` are required for each point's position; `red`, `green`, `blue`, and `alpha` are optional for customizing each point's color.
    """

    data: sequence[uint8] = default_field([])
    """
    Point data, interpreted using `fields`
    """


@dataclass
class PoseInFrame(IdlStruct, typename='foxglove_msgs/PoseInFrame'):
    """
    foxglove_msgs/msg/PoseInFrame
    A timestamped pose for an object or reference frame in 3D space

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of pose
    """

    frame_id: str = ''
    """
    Frame of reference for pose position and orientation
    """

    pose: Pose = default_field(Pose)
    """
    Pose in 3D space
    """


@dataclass
class PosesInFrame(IdlStruct, typename='foxglove_msgs/PosesInFrame'):
    """
    foxglove_msgs/msg/PosesInFrame
    An array of timestamped poses for an object or reference frame in 3D space

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of pose
    """

    frame_id: str = ''
    """
    Frame of reference for pose position and orientation
    """

    poses: sequence[Pose] = default_field([])
    """
    Poses in 3D space
    """


class DeletionAction(Enum):
    MATCHING_ID = 0
    """
    Delete the existing entity on the same topic that has the provided `id`
    """
    ALL = 1
    """
    Delete all existing entities on the same topic
    """


@dataclass
class PrimitiveDeletion(IdlStruct, typename='foxglove_msgs/PrimitiveDeletion'):
    """
    foxglove_msgs/msg/PrimitiveDeletion
    Command to remove previously published markers

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of the marker. Only matching markers earlier than this timestamp will be deleted.
    """

    type: uint8 = 0
    """
    Type of deletion action to perform
    """

    id: str = ''
    """
    Numeric identifier which must match if `kind` is `MATCHING_ID`.
    """


@dataclass
class SceneEntityDeletion(IdlStruct, typename='foxglove_msgs/SceneEntityDeletion'):
    """
    foxglove_msgs/msg/SceneEntityDeletion
    Command to remove previously published entities

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of the deletion. Only matching entities earlier than this timestamp will be deleted.
    """

    type: uint8 = 0
    """
    Type of deletion action to perform
    """

    id: str = ''
    """
    Identifier which must match if `type` is `MATCHING_ID`.
    """


@dataclass
class RawImage(IdlStruct, typename='foxglove_msgs/RawImage'):
    """
    foxglove_msgs/msg/RawImage
    A raw image

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of image
    """

    frame_id: str = ''
    """
    Frame of reference for the image. The origin of the frame is the optical center of the camera. +x points to the right in the image, +y points down, and +z points into the plane of the image.
    """

    width: uint32 = 0
    """
    Image width
    """

    height: uint32 = 0
    """
    Image height
    """

    encoding: str = ''
    """
    Encoding of the raw image data
    
    Supported values: `8UC1`, `8UC3`, `16UC1`, `32FC1`, `bayer_bggr8`, `bayer_gbrg8`, `bayer_grbg8`, `bayer_rggb8`, `bgr8`, `bgra8`, `mono8`, `mono16`, `rgb8`, `rgba8`, `uyvy` or `yuv422`, `yuyv` or `yuv422_yuy2`
    """

    step: uint32 = 0
    """
    Byte length of a single row
    """

    data: sequence[uint8] = default_field([])
    """
    Raw image data
    """


@dataclass
class SpherePrimitive(IdlStruct, typename='foxglove_msgs/SpherePrimitive'):
    """
    foxglove_msgs/msg/SpherePrimitive
    A primitive representing a sphere or ellipsoid

    Generated by https://github.com/foxglove/schemas
    """
    pose: Pose = default_field(Pose)
    """
    Position of the center of the sphere and orientation of the sphere
    """

    size: Vector3 = default_field(Vector3)
    """
    Size (diameter) of the sphere along each axis
    """

    color: Color = default_field(Color)
    """
    Color of the sphere
    """


@dataclass
class TriangleListPrimitive(IdlStruct, typename='foxglove_msgs/TriangleListPrimitive'):
    """
    foxglove_msgs/msg/TriangleListPrimitive
    A primitive representing a set of triangles or a surface tiled by triangles

    Generated by https://github.com/foxglove/schemas
    """
    pose: Pose = default_field(Pose)
    """
    Origin of triangles relative to reference frame
    """

    points: sequence[Point] = default_field([])
    """
    Vertices to use for triangles, interpreted as a list of triples (0-1-2, 3-4-5, ...)
    """

    color: Color = default_field(Color)
    """
    Solid color to use for the whole shape. One of `color` or `colors` must be provided.
    """

    colors: sequence[Color] = default_field([])
    """
    Per-vertex colors (if specified, must have the same length as `points`). One of `color` or `colors` must be provided.
    """

    indices: sequence[uint32] = default_field([])
    """
    Indices into the `points` and `colors` attribute arrays, which can be used to avoid duplicating attribute data.
    
    If omitted or empty, indexing will not be used. This default behavior is equivalent to specifying [0, 1, ..., N-1] for the indices (where N is the number of `points` provided).
    """


@dataclass
class TextPrimitive(IdlStruct, typename='foxglove_msgs/TextPrimitive'):
    """
    foxglove_msgs/msg/TextPrimitive
    A primitive representing a text label

    Generated by https://github.com/foxglove/schemas
    """
    pose: Pose = default_field(Pose)
    """
    Position of the center of the text box and orientation of the text. Identity orientation means the text is oriented in the xy-plane and flows from -x to +x.
    """

    billboard: bool = False
    """
    Whether the text should respect `pose.orientation` (false) or always face the camera (true)
    """

    font_size: float64 = 0
    """
    Font size (height of one line of text)
    """

    scale_invariant: bool = False
    """
    Indicates whether `font_size` is a fixed size in screen pixels (true), or specified in world coordinates and scales with distance from the camera (false)
    """

    color: Color = default_field(Color)
    """
    Color of the text
    """

    text: str = ''
    """
    Text
    """


@dataclass
class SceneEntity(IdlStruct, typename='foxglove_msgs/SceneEntity'):
    """
    foxglove_msgs/msg/SceneEntity
    A visual element in a 3D scene. An entity may be composed of multiple primitives which all share the same frame of reference.

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of the entity
    """

    frame_id: str = ''
    """
    Frame of reference
    """

    id: str = ''
    """
    Identifier for the entity. A entity will replace any prior entity on the same topic with the same `id`.
    """

    lifetime: Duration = default_field(Duration)
    """
    Length of time (relative to `timestamp`) after which the entity should be automatically removed. Zero value indicates the entity should remain visible until it is replaced or deleted.
    """

    frame_locked: bool = False
    """
    Whether the entity should keep its location in the fixed frame (false) or follow the frame specified in `frame_id` as it moves relative to the fixed frame (true)
    """

    metadata: sequence[KeyValuePair] = default_field([])
    """
    Additional user-provided metadata associated with the entity. Keys must be unique.
    """

    arrows: sequence[ArrowPrimitive] = default_field([])
    """
    Arrow primitives
    """

    cubes: sequence[CubePrimitive] = default_field([])
    """
    Cube primitives
    """

    spheres: sequence[SpherePrimitive] = default_field([])
    """
    Sphere primitives
    """

    cylinders: sequence[CylinderPrimitive] = default_field([])
    """
    Cylinder primitives
    """

    lines: sequence[LinePrimitive] = default_field([])
    """
    Line primitives
    """

    triangles: sequence[TriangleListPrimitive] = default_field([])
    """
    Triangle list primitives
    """

    texts: sequence[TextPrimitive] = default_field([])
    """
    Text primitives
    """

    models: sequence[ModelPrimitive] = default_field([])
    """
    Model primitives
    """


@dataclass
class SceneEntities(IdlStruct, typename='foxglove_msgs/SceneEntities'):
    """
    foxglove_msgs/msg/SceneEntities
    An update to the entities displayed in a 3D scene

    Generated by https://github.com/foxglove/schemas
    """
    deletions: sequence[SceneEntityDeletion] = default_field([])
    """
    Scene entities to delete
    """

    entities: sequence[SceneEntity] = default_field([])
    """
    Scene entities to add or replace
    """


@dataclass
class SceneEntityUpdate(IdlStruct, typename='foxglove_msgs/SceneEntityUpdate'):
    """
    foxglove_msgs/msg/SceneEntityUpdate
    An update to the entities displayed in a 3D scene

    Generated by https://github.com/foxglove/schemas
    """
    deletions: sequence[SceneEntityDeletion] = default_field([])
    """
    Scene entities to delete
    """

    entities: sequence[SceneEntity] = default_field([])
    """
    Scene entities to add or replace
    """


@dataclass
class SceneUpdate(IdlStruct, typename='foxglove_msgs/SceneUpdate'):
    """
    foxglove_msgs/msg/SceneUpdate
    An update to the entities displayed in a 3D scene

    Generated by https://github.com/foxglove/schemas
    """
    deletions: sequence[SceneEntityDeletion] = default_field([])
    """
    Scene entities to delete
    """

    entities: sequence[SceneEntity] = default_field([])
    """
    Scene entities to add or replace
    """


@dataclass
class SphereMarker(IdlStruct, typename='foxglove_msgs/SphereMarker'):
    """
    foxglove_msgs/msg/SphereMarker
    A marker representing a sphere or ellipsoid

    Generated by https://github.com/foxglove/schemas
    """
    pose: Pose = default_field(Pose)
    """
    Position of the center of the sphere and orientation of the sphere
    """

    size: Vector3 = default_field(Vector3)
    """
    Size (diameter) of the sphere along each axis
    """

    color: Color = default_field(Color)
    """
    Color of the sphere
    """


@dataclass
class Transform(IdlStruct, typename='foxglove_msgs/Transform'):
    """
    Generated from Transform by @foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Transform time
    """

    translation: Vector3 = default_field(Vector3)
    """
    Translation component of the transform
    """

    rotation: Quaternion = default_field(Quaternion)
    """
    Rotation component of the transform
    """


@dataclass
class TrianglesMarker(IdlStruct, typename='foxglove_msgs/TrianglesMarker'):
    """
    foxglove_msgs/msg/TrianglesMarker
    A marker representing a set of triangles or a surface tiled by triangles

    Generated by https://github.com/foxglove/schemas
    """
    timestamp: Time = default_field(Time)
    """
    Timestamp of the marker
    """

    frame_id: str = ''
    """
    Frame of reference
    """

    id: str = ''
    """
    Identifier for the marker. A marker will replace any prior marker on the same topic with the same `id`.
    """

    lifetime: Duration = default_field(Duration)
    """
    Length of time (relative to `timestamp`) after which the marker should be automatically removed. Zero value indicates the marker should remain visible until it is replaced or deleted.
    """

    frame_locked: bool = False
    """
    Whether the marker should keep its location in the fixed frame (false) or follow the frame specified in `frame_id` as it moves relative to the fixed frame (true)
    """

    metadata: sequence[KeyValuePair] = default_field([])
    """
    Additional user-provided metadata associated with the marker. Keys must be unique.
    """

    pose: Pose = default_field(Pose)
    """
    Origin of triangles relative to reference frame
    """

    points: sequence[Point] = default_field([])
    """
    Vertices to use for triangles, interpreted as a list of triples (0-1-2, 3-4-5, ...)
    """

    color: Color = default_field(Color)
    """
    Solid color to use for the whole shape. One of `color` or `colors` must be provided.
    """

    colors: sequence[Color] = default_field([])
    """
    Per-vertex colors (if specified, must have the same length as `points`). One of `color` or `colors` should be provided.
    """

    indices: sequence[uint32] = default_field([])
    """
    Indices into the `points` and `colors` attribute arrays, which can be used to avoid duplicating attribute data.
    
    If omitted or empty, indexing will not be used. This default behavior is equivalent to specifying [0, 1, ..., N-1] for the indices (where N is the number of `points` provided).
    """


# Schema registry support
_TYPES = {
    "ArrowMarker": ArrowMarker,
    "CircleAnnotation": CircleAnnotation,
    "Color": Color,
    "CompressedVideo": CompressedVideo,
    "CubeMarker": CubeMarker,
    "CylinderMarker": CylinderMarker,
    "ImageAnnotations": ImageAnnotations,
    "KeyValuePair": KeyValuePair,
    "LineMarker": LineMarker,
    "ModelMarker": ModelMarker,
    "Point2": Point2,
    "PointsAnnotation": PointsAnnotation,
    "SceneEntity": SceneEntity,
    "SceneEntityDeletion": SceneEntityDeletion,
    "SceneUpdate": SceneUpdate,
    "SphereMarker": SphereMarker,
    "TextAnnotation": TextAnnotation,
    "TextMarker": TextMarker,
    "TriangleListMarker": TriangleListMarker,
}


def is_type_supported(type_name: str) -> bool:
    """Check if a type name is supported by this module."""
    return type_name in _TYPES


def list_types() -> list[str]:
    """List all type schema names in this module."""
    return [f"foxglove_msgs/msg/{name}" for name in sorted(_TYPES.keys())]


def get_type(type_name: str) -> type:
    """Get the type class by name. Returns None if not found."""
    return _TYPES.get(type_name)



# SPDX-License-Identifier: Apache-2.0
# Copyright © 2025 Au-Zone Technologies. All Rights Reserved.

"""
Schema Registry for EdgeFirst Schemas.

This module provides functions to work with ROS2-style schema names:
- `from_schema()`: Get the message class for a schema name
- `schema_name()`: Get the schema name for a message class or instance

Schema names follow the ROS2 convention: `package/msg/TypeName`
(e.g., `sensor_msgs/msg/Image`, `geometry_msgs/msg/Pose`)

Example usage:

    from edgefirst.schemas import from_schema, schema_name, sensor_msgs

    # Get class from schema name
    ImageClass = from_schema("sensor_msgs/msg/Image")
    img = ImageClass()

    # Get schema name from class or instance
    name = schema_name(sensor_msgs.Image)  # "sensor_msgs/msg/Image"
    name = schema_name(img)                 # "sensor_msgs/msg/Image"
"""

from typing import Union

# Import all schema modules
from . import (
    builtin_interfaces,
    edgefirst_msgs,
    foxglove_msgs,
    geometry_msgs,
    nav_msgs,
    sensor_msgs,
    std_msgs,
)

# Type alias for schema classes
SchemaClass = type

# Registry mapping package names to their modules
_PACKAGE_MODULES = {
    "builtin_interfaces": builtin_interfaces,
    "edgefirst_msgs": edgefirst_msgs,
    "foxglove_msgs": foxglove_msgs,
    "geometry_msgs": geometry_msgs,
    "nav_msgs": nav_msgs,
    "sensor_msgs": sensor_msgs,
    "std_msgs": std_msgs,
}


def from_schema(schema: str) -> SchemaClass:
    """
    Get the message class for a given schema name.

    Args:
        schema: Schema name in ROS2 format (e.g., "sensor_msgs/msg/Image")

    Returns:
        The message class corresponding to the schema name

    Raises:
        ValueError: If the schema name format is invalid
        KeyError: If the package or type is not found

    Examples:
        >>> ImageClass = from_schema("sensor_msgs/msg/Image")
        >>> img = ImageClass()

        >>> PoseClass = from_schema("geometry_msgs/msg/Pose")
    """
    parts = schema.split("/")

    if len(parts) == 3:
        # Standard ROS2 format: package/msg/TypeName
        package, msg_marker, type_name = parts
        if msg_marker != "msg":
            raise ValueError(
                f"Invalid schema format '{schema}': "
                f"expected 'msg' but got '{msg_marker}'"
            )
    elif len(parts) == 2:
        # Short format: package/TypeName (used by pycdr2 typename)
        package, type_name = parts
    else:
        raise ValueError(
            f"Invalid schema format '{schema}': "
            f"expected 'package/msg/TypeName' or 'package/TypeName'"
        )

    # Look up the package module
    if package not in _PACKAGE_MODULES:
        raise KeyError(f"Unknown package '{package}' in schema '{schema}'")

    module = _PACKAGE_MODULES[package]

    # Use the module's get_type() function if available (hierarchical dispatch)
    if hasattr(module, 'get_type'):
        cls = module.get_type(type_name)
        if cls is not None:
            return cls
        raise KeyError(f"Unknown type '{type_name}' in package '{package}'")

    # Fallback: Look up the type directly in the module
    if not hasattr(module, type_name):
        raise KeyError(f"Unknown type '{type_name}' in package '{package}'")

    return getattr(module, type_name)


def schema_name(cls_or_instance: Union[SchemaClass, object]) -> str:
    """
    Get the ROS2 schema name for a message class or instance.

    Args:
        cls_or_instance: A message class or message instance

    Returns:
        Schema name in ROS2 format (e.g., "sensor_msgs/msg/Image")

    Raises:
        TypeError: If the argument is not a schema class or instance
        ValueError: If the class doesn't have a typename attribute

    Examples:
        >>> from edgefirst.schemas import sensor_msgs
        >>> schema_name(sensor_msgs.Image)
        'sensor_msgs/msg/Image'
        >>> img = sensor_msgs.Image()
        >>> schema_name(img)
        'sensor_msgs/msg/Image'
    """
    # Get the class if we have an instance
    if isinstance(cls_or_instance, type):
        cls = cls_or_instance
    else:
        cls = type(cls_or_instance)

    # Get the typename from pycdr2 IdlStruct metadata
    # pycdr2 stores typename in __idl_typename__ attribute
    if hasattr(cls, "__idl_typename__"):
        typename = cls.__idl_typename__
    else:
        raise ValueError(
            f"Class '{cls.__name__}' does not have a typename attribute. "
            "Ensure it inherits from IdlStruct with typename parameter."
        )

    # Convert pycdr2 format (package/TypeName) to ROS2 format (package/msg/Type)
    if "/" in typename:
        parts = typename.split("/")
        if len(parts) == 2:
            package, type_name = parts
            return f"{package}/msg/{type_name}"
        elif len(parts) == 3 and parts[1] == "msg":
            # Already in ROS2 format
            return typename
        else:
            raise ValueError(f"Unexpected typename format: '{typename}'")
    else:
        # No package prefix - try to infer from module
        module_name = cls.__module__
        if module_name.startswith("edgefirst.schemas."):
            package = module_name.split(".")[-1]
            return f"{package}/msg/{typename}"
        else:
            raise ValueError(
                f"Cannot determine package for type '{typename}' "
                f"from module '{module_name}'"
            )


def list_schemas(package: str = None) -> list[str]:
    """
    List all available schema names.

    Args:
        package: Optional package name to filter by (e.g., "sensor_msgs")

    Returns:
        List of schema names in ROS2 format

    Examples:
        >>> schemas = list_schemas("sensor_msgs")
        >>> "sensor_msgs/msg/Image" in schemas
        True
    """
    schemas = []
    packages = [package] if package else _PACKAGE_MODULES.keys()

    for pkg in packages:
        if pkg not in _PACKAGE_MODULES:
            continue

        module = _PACKAGE_MODULES[pkg]

        # Use the module's list_types() function if available
        if hasattr(module, 'list_types'):
            schemas.extend(module.list_types())
        else:
            # Fallback: iterate over module attributes
            for name in dir(module):
                obj = getattr(module, name)
                if isinstance(obj, type) and hasattr(obj, "__idl_typename__"):
                    try:
                        sname = schema_name(obj)
                        # Only include if schema belongs to module's package
                        if sname.startswith(f"{pkg}/"):
                            schemas.append(sname)
                    except ValueError:
                        pass  # Skip types without valid typename

    return sorted(set(schemas))  # Use set to deduplicate


def is_supported(schema: str) -> bool:
    """
    Check if a schema name is supported by this library.

    Args:
        schema: Schema name in ROS2 format (e.g., "sensor_msgs/msg/Image")

    Returns:
        True if the schema is supported, False otherwise

    Examples:
        >>> is_supported("sensor_msgs/msg/Image")
        True
        >>> is_supported("unknown_msgs/msg/Foo")
        False
    """
    parts = schema.split("/")

    if len(parts) == 3:
        package, msg_marker, type_name = parts
        if msg_marker != "msg":
            return False
    elif len(parts) == 2:
        package, type_name = parts
    else:
        return False

    if package not in _PACKAGE_MODULES:
        return False

    module = _PACKAGE_MODULES[package]

    # Use the module's is_type_supported() function if available
    if hasattr(module, 'is_type_supported'):
        return module.is_type_supported(type_name)

    # Fallback: check if attribute exists
    return hasattr(module, type_name)
